# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: common/policy.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.common import authorization_pb2 as common_dot_authorization__pb2
from union.internal.common import identifier_pb2 as common_dot_identifier__pb2
from union.internal.validate.validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13\x63ommon/policy.proto\x12\x0f\x63loudidl.common\x1a\x1a\x63ommon/authorization.proto\x1a\x17\x63ommon/identifier.proto\x1a\x17validate/validate.proto\"\xa3\x01\n\x06Policy\x12;\n\x02id\x18\x01 \x01(\x0b\x32!.cloudidl.common.PolicyIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x02id\x12:\n\x08\x62indings\x18\x02 \x03(\x0b\x32\x1e.cloudidl.common.PolicyBindingR\x08\x62indings\x12 \n\x0b\x64\x65scription\x18\x03 \x01(\tR\x0b\x64\x65scription\"\x94\x01\n\rPolicyBinding\x12\x42\n\x07role_id\x18\x01 \x01(\x0b\x32\x1f.cloudidl.common.RoleIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x06roleId\x12?\n\x08resource\x18\x02 \x01(\x0b\x32\x19.cloudidl.common.ResourceB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x08resourceB\xac\x01\n\x13\x63om.cloudidl.commonB\x0bPolicyProtoH\x02P\x01Z)github.com/unionai/cloud/gen/pb-go/common\xa2\x02\x03\x43\x43X\xaa\x02\x0f\x43loudidl.Common\xca\x02\x0f\x43loudidl\\Common\xe2\x02\x1b\x43loudidl\\Common\\GPBMetadata\xea\x02\x10\x43loudidl::Commonb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'common.policy_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023com.cloudidl.commonB\013PolicyProtoH\002P\001Z)github.com/unionai/cloud/gen/pb-go/common\242\002\003CCX\252\002\017Cloudidl.Common\312\002\017Cloudidl\\Common\342\002\033Cloudidl\\Common\\GPBMetadata\352\002\020Cloudidl::Common'
  _POLICY.fields_by_name['id']._options = None
  _POLICY.fields_by_name['id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _POLICYBINDING.fields_by_name['role_id']._options = None
  _POLICYBINDING.fields_by_name['role_id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _POLICYBINDING.fields_by_name['resource']._options = None
  _POLICYBINDING.fields_by_name['resource']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_POLICY']._serialized_start=119
  _globals['_POLICY']._serialized_end=282
  _globals['_POLICYBINDING']._serialized_start=285
  _globals['_POLICYBINDING']._serialized_end=433
# @@protoc_insertion_point(module_scope)



from union.app._arize import ArizeConfig, PhoenixConfig
from union.app._models import (
    App,
    FlyteConnectorApp,
    Input,
    Link,
    ScalingMetric,
    URLQuery,
)
from union.app._weave import WeaveConfig

__all__ = [
    "App",
    "ArizeConfig",
    "FlyteConnectorApp",
    "Input",
    "Link",
    "PhoenixConfig",
    "ScalingMetric",
    "URLQuery",
    "WeaveConfig",
]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: common/authorization.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.common import identifier_pb2 as common_dot_identifier__pb2
from union.internal.validate.validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1a\x63ommon/authorization.proto\x12\x0f\x63loudidl.common\x1a\x17\x63ommon/identifier.proto\x1a\x17validate/validate.proto\"+\n\x0cOrganization\x12\x1b\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x04name\"i\n\x06\x44omain\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12K\n\x0corganization\x18\x02 \x01(\x0b\x32\x1d.cloudidl.common.OrganizationB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x0corganization\"a\n\x07Project\x12\x1b\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x04name\x12\x39\n\x06\x64omain\x18\x02 \x01(\x0b\x32\x17.cloudidl.common.DomainB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x06\x64omain\"e\n\x08Workflow\x12\x1b\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x04name\x12<\n\x07project\x18\x02 \x01(\x0b\x32\x18.cloudidl.common.ProjectB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x07project\"g\n\nLaunchPlan\x12\x1b\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x04name\x12<\n\x07project\x18\x02 \x01(\x0b\x32\x18.cloudidl.common.ProjectB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x07project\"\xfd\x02\n\x08Resource\x12\x43\n\x0corganization\x18\x01 \x01(\x0b\x32\x1d.cloudidl.common.OrganizationH\x00R\x0corganization\x12\x31\n\x06\x64omain\x18\x02 \x01(\x0b\x32\x17.cloudidl.common.DomainH\x00R\x06\x64omain\x12\x34\n\x07project\x18\x03 \x01(\x0b\x32\x18.cloudidl.common.ProjectH\x00R\x07project\x12\x37\n\x08workflow\x18\x04 \x01(\x0b\x32\x19.cloudidl.common.WorkflowH\x00R\x08workflow\x12>\n\x0blaunch_plan\x18\x05 \x01(\x0b\x32\x1b.cloudidl.common.LaunchPlanH\x00R\nlaunchPlan\x12>\n\x07\x63luster\x18\x06 \x01(\x0b\x32\".cloudidl.common.ClusterIdentifierH\x00R\x07\x63lusterB\n\n\x08resource\"v\n\nPermission\x12\x35\n\x08resource\x18\x01 \x01(\x0b\x32\x19.cloudidl.common.ResourceR\x08resource\x12\x31\n\x07\x61\x63tions\x18\x02 \x03(\x0e\x32\x17.cloudidl.common.ActionR\x07\x61\x63tions*\xf4\x03\n\x06\x41\x63tion\x12\x0f\n\x0b\x41\x43TION_NONE\x10\x00\x12\x15\n\rACTION_CREATE\x10\x01\x1a\x02\x08\x01\x12\x13\n\x0b\x41\x43TION_READ\x10\x02\x1a\x02\x08\x01\x12\x15\n\rACTION_UPDATE\x10\x03\x1a\x02\x08\x01\x12\x15\n\rACTION_DELETE\x10\x04\x1a\x02\x08\x01\x12\x1f\n\x1b\x41\x43TION_VIEW_FLYTE_INVENTORY\x10\x05\x12 \n\x1c\x41\x43TION_VIEW_FLYTE_EXECUTIONS\x10\x06\x12#\n\x1f\x41\x43TION_REGISTER_FLYTE_INVENTORY\x10\x07\x12\"\n\x1e\x41\x43TION_CREATE_FLYTE_EXECUTIONS\x10\x08\x12\x1d\n\x19\x41\x43TION_ADMINISTER_PROJECT\x10\t\x12\x1d\n\x19\x41\x43TION_MANAGE_PERMISSIONS\x10\n\x12\x1d\n\x19\x41\x43TION_ADMINISTER_ACCOUNT\x10\x0b\x12\x19\n\x15\x41\x43TION_MANAGE_CLUSTER\x10\x0c\x12,\n(ACTION_EDIT_EXECUTION_RELATED_ATTRIBUTES\x10\r\x12*\n&ACTION_EDIT_CLUSTER_RELATED_ATTRIBUTES\x10\x0e\x12!\n\x1d\x41\x43TION_EDIT_UNUSED_ATTRIBUTES\x10\x0f\x42\xb3\x01\n\x13\x63om.cloudidl.commonB\x12\x41uthorizationProtoH\x02P\x01Z)github.com/unionai/cloud/gen/pb-go/common\xa2\x02\x03\x43\x43X\xaa\x02\x0f\x43loudidl.Common\xca\x02\x0f\x43loudidl\\Common\xe2\x02\x1b\x43loudidl\\Common\\GPBMetadata\xea\x02\x10\x43loudidl::Commonb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'common.authorization_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023com.cloudidl.commonB\022AuthorizationProtoH\002P\001Z)github.com/unionai/cloud/gen/pb-go/common\242\002\003CCX\252\002\017Cloudidl.Common\312\002\017Cloudidl\\Common\342\002\033Cloudidl\\Common\\GPBMetadata\352\002\020Cloudidl::Common'
  _ACTION.values_by_name["ACTION_CREATE"]._options = None
  _ACTION.values_by_name["ACTION_CREATE"]._serialized_options = b'\010\001'
  _ACTION.values_by_name["ACTION_READ"]._options = None
  _ACTION.values_by_name["ACTION_READ"]._serialized_options = b'\010\001'
  _ACTION.values_by_name["ACTION_UPDATE"]._options = None
  _ACTION.values_by_name["ACTION_UPDATE"]._serialized_options = b'\010\001'
  _ACTION.values_by_name["ACTION_DELETE"]._options = None
  _ACTION.values_by_name["ACTION_DELETE"]._serialized_options = b'\010\001'
  _ORGANIZATION.fields_by_name['name']._options = None
  _ORGANIZATION.fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _DOMAIN.fields_by_name['organization']._options = None
  _DOMAIN.fields_by_name['organization']._serialized_options = b'\372B\005\212\001\002\020\001'
  _PROJECT.fields_by_name['name']._options = None
  _PROJECT.fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _PROJECT.fields_by_name['domain']._options = None
  _PROJECT.fields_by_name['domain']._serialized_options = b'\372B\005\212\001\002\020\001'
  _WORKFLOW.fields_by_name['name']._options = None
  _WORKFLOW.fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _WORKFLOW.fields_by_name['project']._options = None
  _WORKFLOW.fields_by_name['project']._serialized_options = b'\372B\005\212\001\002\020\001'
  _LAUNCHPLAN.fields_by_name['name']._options = None
  _LAUNCHPLAN.fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _LAUNCHPLAN.fields_by_name['project']._options = None
  _LAUNCHPLAN.fields_by_name['project']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_ACTION']._serialized_start=1061
  _globals['_ACTION']._serialized_end=1561
  _globals['_ORGANIZATION']._serialized_start=97
  _globals['_ORGANIZATION']._serialized_end=140
  _globals['_DOMAIN']._serialized_start=142
  _globals['_DOMAIN']._serialized_end=247
  _globals['_PROJECT']._serialized_start=249
  _globals['_PROJECT']._serialized_end=346
  _globals['_WORKFLOW']._serialized_start=348
  _globals['_WORKFLOW']._serialized_end=449
  _globals['_LAUNCHPLAN']._serialized_start=451
  _globals['_LAUNCHPLAN']._serialized_end=554
  _globals['_RESOURCE']._serialized_start=557
  _globals['_RESOURCE']._serialized_end=938
  _globals['_PERMISSION']._serialized_start=940
  _globals['_PERMISSION']._serialized_end=1058
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from union.internal.workspace import workspace_instance_payload_pb2 as workspace_dot_workspace__instance__payload__pb2


class WorkspaceInstanceServiceStub(object):
    """WorkspaceRegistryService defines the inventory service for creating and fetching workspace definitions.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.StartWorkspaceInstance = channel.unary_unary(
                '/cloudidl.workspace.WorkspaceInstanceService/StartWorkspaceInstance',
                request_serializer=workspace_dot_workspace__instance__payload__pb2.StartWorkspaceInstanceRequest.SerializeToString,
                response_deserializer=workspace_dot_workspace__instance__payload__pb2.StartWorkspaceInstanceResponse.FromString,
                )
        self.GetWorkspaceInstance = channel.unary_unary(
                '/cloudidl.workspace.WorkspaceInstanceService/GetWorkspaceInstance',
                request_serializer=workspace_dot_workspace__instance__payload__pb2.GetWorkspaceInstanceRequest.SerializeToString,
                response_deserializer=workspace_dot_workspace__instance__payload__pb2.GetWorkspaceInstanceResponse.FromString,
                )
        self.ListWorkspaceInstances = channel.unary_unary(
                '/cloudidl.workspace.WorkspaceInstanceService/ListWorkspaceInstances',
                request_serializer=workspace_dot_workspace__instance__payload__pb2.ListWorkspaceInstancesRequest.SerializeToString,
                response_deserializer=workspace_dot_workspace__instance__payload__pb2.ListWorkspaceInstancesResponse.FromString,
                )
        self.StopWorkspaceInstance = channel.unary_unary(
                '/cloudidl.workspace.WorkspaceInstanceService/StopWorkspaceInstance',
                request_serializer=workspace_dot_workspace__instance__payload__pb2.StopWorkspaceInstanceRequest.SerializeToString,
                response_deserializer=workspace_dot_workspace__instance__payload__pb2.StopWorkspaceInstanceResponse.FromString,
                )
        self.WatchWorkspaceInstances = channel.unary_stream(
                '/cloudidl.workspace.WorkspaceInstanceService/WatchWorkspaceInstances',
                request_serializer=workspace_dot_workspace__instance__payload__pb2.WatchWorkspaceInstancesRequest.SerializeToString,
                response_deserializer=workspace_dot_workspace__instance__payload__pb2.WatchWorkspaceInstancesResponse.FromString,
                )


class WorkspaceInstanceServiceServicer(object):
    """WorkspaceRegistryService defines the inventory service for creating and fetching workspace definitions.
    """

    def StartWorkspaceInstance(self, request, context):
        """StartWorkspaceInstance creates a new workspace definition.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetWorkspaceInstance(self, request, context):
        """GetWorkspaceInstance fetches a workspace definition by its ID.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListWorkspaceInstances(self, request, context):
        """ListWorkspaceInstances fetches all workspace definitions for a given organization and project.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StopWorkspaceInstance(self, request, context):
        """StopWorkspaceInstance fetches a workspace definition by its ID.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WatchWorkspaceInstances(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_WorkspaceInstanceServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'StartWorkspaceInstance': grpc.unary_unary_rpc_method_handler(
                    servicer.StartWorkspaceInstance,
                    request_deserializer=workspace_dot_workspace__instance__payload__pb2.StartWorkspaceInstanceRequest.FromString,
                    response_serializer=workspace_dot_workspace__instance__payload__pb2.StartWorkspaceInstanceResponse.SerializeToString,
            ),
            'GetWorkspaceInstance': grpc.unary_unary_rpc_method_handler(
                    servicer.GetWorkspaceInstance,
                    request_deserializer=workspace_dot_workspace__instance__payload__pb2.GetWorkspaceInstanceRequest.FromString,
                    response_serializer=workspace_dot_workspace__instance__payload__pb2.GetWorkspaceInstanceResponse.SerializeToString,
            ),
            'ListWorkspaceInstances': grpc.unary_unary_rpc_method_handler(
                    servicer.ListWorkspaceInstances,
                    request_deserializer=workspace_dot_workspace__instance__payload__pb2.ListWorkspaceInstancesRequest.FromString,
                    response_serializer=workspace_dot_workspace__instance__payload__pb2.ListWorkspaceInstancesResponse.SerializeToString,
            ),
            'StopWorkspaceInstance': grpc.unary_unary_rpc_method_handler(
                    servicer.StopWorkspaceInstance,
                    request_deserializer=workspace_dot_workspace__instance__payload__pb2.StopWorkspaceInstanceRequest.FromString,
                    response_serializer=workspace_dot_workspace__instance__payload__pb2.StopWorkspaceInstanceResponse.SerializeToString,
            ),
            'WatchWorkspaceInstances': grpc.unary_stream_rpc_method_handler(
                    servicer.WatchWorkspaceInstances,
                    request_deserializer=workspace_dot_workspace__instance__payload__pb2.WatchWorkspaceInstancesRequest.FromString,
                    response_serializer=workspace_dot_workspace__instance__payload__pb2.WatchWorkspaceInstancesResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'cloudidl.workspace.WorkspaceInstanceService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class WorkspaceInstanceService(object):
    """WorkspaceRegistryService defines the inventory service for creating and fetching workspace definitions.
    """

    @staticmethod
    def StartWorkspaceInstance(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.workspace.WorkspaceInstanceService/StartWorkspaceInstance',
            workspace_dot_workspace__instance__payload__pb2.StartWorkspaceInstanceRequest.SerializeToString,
            workspace_dot_workspace__instance__payload__pb2.StartWorkspaceInstanceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetWorkspaceInstance(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.workspace.WorkspaceInstanceService/GetWorkspaceInstance',
            workspace_dot_workspace__instance__payload__pb2.GetWorkspaceInstanceRequest.SerializeToString,
            workspace_dot_workspace__instance__payload__pb2.GetWorkspaceInstanceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListWorkspaceInstances(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.workspace.WorkspaceInstanceService/ListWorkspaceInstances',
            workspace_dot_workspace__instance__payload__pb2.ListWorkspaceInstancesRequest.SerializeToString,
            workspace_dot_workspace__instance__payload__pb2.ListWorkspaceInstancesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StopWorkspaceInstance(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.workspace.WorkspaceInstanceService/StopWorkspaceInstance',
            workspace_dot_workspace__instance__payload__pb2.StopWorkspaceInstanceRequest.SerializeToString,
            workspace_dot_workspace__instance__payload__pb2.StopWorkspaceInstanceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WatchWorkspaceInstances(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/cloudidl.workspace.WorkspaceInstanceService/WatchWorkspaceInstances',
            workspace_dot_workspace__instance__payload__pb2.WatchWorkspaceInstancesRequest.SerializeToString,
            workspace_dot_workspace__instance__payload__pb2.WatchWorkspaceInstancesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: app/app_logs_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.app import app_logs_payload_pb2 as app_dot_app__logs__payload__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1a\x61pp/app_logs_service.proto\x12\x0c\x63loudidl.app\x1a\x1a\x61pp/app_logs_payload.proto2b\n\x0e\x41ppLogsService\x12P\n\x08TailLogs\x12\x1d.cloudidl.app.TailLogsRequest\x1a\x1e.cloudidl.app.TailLogsResponse\"\x03\x90\x02\x01\x30\x01\x42\xa2\x01\n\x10\x63om.cloudidl.appB\x13\x41ppLogsServiceProtoH\x02P\x01Z&github.com/unionai/cloud/gen/pb-go/app\xa2\x02\x03\x43\x41X\xaa\x02\x0c\x43loudidl.App\xca\x02\x0c\x43loudidl\\App\xe2\x02\x18\x43loudidl\\App\\GPBMetadata\xea\x02\rCloudidl::Appb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'app.app_logs_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\020com.cloudidl.appB\023AppLogsServiceProtoH\002P\001Z&github.com/unionai/cloud/gen/pb-go/app\242\002\003CAX\252\002\014Cloudidl.App\312\002\014Cloudidl\\App\342\002\030Cloudidl\\App\\GPBMetadata\352\002\rCloudidl::App'
  _APPLOGSSERVICE.methods_by_name['TailLogs']._options = None
  _APPLOGSSERVICE.methods_by_name['TailLogs']._serialized_options = b'\220\002\001'
  _globals['_APPLOGSSERVICE']._serialized_start=72
  _globals['_APPLOGSSERVICE']._serialized_end=170
# @@protoc_insertion_point(module_scope)



import base64
import json
import logging
import os
import subprocess
import tempfile
from pathlib import Path
from typing import List, Optional, Tuple

logger = logging.getLogger("union.cli.ucimage._docker_credentials_helper")

_CONFIG_JSON = "config.json"
_DEFAULT_CONFIG_PATH = f"~/.docker/{_CONFIG_JSON}"
_CRED_HELPERS = "credHelpers"
_CREDS_STORE = "credsStore"


def _load_docker_config(config_path: str = _DEFAULT_CONFIG_PATH) -> dict:
    """Load Docker config from specified path

    Raises:
        FileNotFoundError: If the config file does not exist
        json.JSONDecodeError: If the config file is not valid JSON
    """
    if not config_path:
        config_path = _DEFAULT_CONFIG_PATH

    config_path = os.path.expanduser(config_path)
    with open(config_path) as f:
        return json.load(f)


def _get_credential_helper(config: dict, registry: Optional[str] = None) -> Optional[str]:
    """Get credential helper for registry or global default"""
    if registry and _CRED_HELPERS in config and registry in config[_CRED_HELPERS]:
        return config[_CRED_HELPERS].get(registry)
    credsStore = config.get(_CREDS_STORE)
    return credsStore


def _get_credentials(helper: str, registry: str) -> Optional[Tuple[str, str]]:
    """Get credentials from system credential helper"""

    # Naming convention established.
    helper_cmd = f"docker-credential-{helper}"

    try:
        process = subprocess.Popen(
            [helper_cmd, "get"], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True
        )
        output, error = process.communicate(input=registry)

        if process.returncode != 0:
            logging.error(f"Credential helper error: {error}")
            return None

        creds = json.loads(output)
        return creds.get("Username"), creds.get("Secret")
    except FileNotFoundError:
        logger.error(f"Credential helper {helper_cmd} not found in PATH")
        return None
    except Exception as e:
        logger.error(f"Error getting credentials: {e!s}")
        return None


def derive_docker_credentials(
    registries: Optional[List[str]], docker_config_path: Optional[Path], output_path: Optional[Path]
) -> Path:
    """
    Create new Docker config file. Targets specific registries if provided, otherwise all registries
    in auths are targeted.

    This function extracts Docker registry credentials from the user's Docker config file
    and creates a new config file containing only the credentials for the specified registries.
    It handles credentials stored directly in the config file as well as those managed by
    credential helpers.

    Args:
        registries: List of registries to extract credentials for. If None, all registries
                   from the config will be used.
        docker_config_path: Path to the Docker config file. If None, the function will look
                          for the config file in the standard locations: first checking the
                          DOCKER_CONFIG environment variable, then falling back to ~/.docker/config.json.
        output_path: Path where the new config file should be written. If None, a temporary
                    directory will be created and the config will be written to
                    {temp_dir}/docker/config.json.

    Returns:
        Path: The path to the newly created Docker config file containing only the
              extracted credentials.
    """

    # Check DOCKER_CONFIG environment variable first
    if not docker_config_path:
        docker_config_env = os.environ.get("DOCKER_CONFIG")
        if docker_config_env:
            docker_config_path = os.path.join(docker_config_env, _CONFIG_JSON)
        else:
            docker_config_path = os.path.expanduser(_DEFAULT_CONFIG_PATH)

    config = _load_docker_config(docker_config_path)

    # Create new config structure with empty auths
    new_config = {"auths": {}}

    registries = registries or config.get("auths", {}).keys()

    for registry in registries:
        registry_config = config.get("auths", {}).get(registry, {})
        if registry_config.get("auth"):
            new_config["auths"][registry] = registry_config.copy()
        else:
            # Try to get credentials from helper
            helper = _get_credential_helper(config, registry)
            if helper:
                creds = _get_credentials(helper, registry)
                if creds:
                    username, password = creds
                    auth_string = f"{username}:{password}"
                    new_config["auths"][registry] = {"auth": base64.b64encode(auth_string.encode()).decode()}

    # Save the new config to the output path or a temporary file
    if not output_path:
        temp_dir = tempfile.mkdtemp()
        docker_dir = os.path.join(temp_dir, "docker")
        os.makedirs(docker_dir, exist_ok=True)
        output_path = os.path.join(docker_dir, _CONFIG_JSON)

    with open(output_path, "w") as f:
        json.dump(new_config, f)

    return output_path



import re
from pathlib import Path
from typing import List

import rich_click as click
import yaml
from rich.console import Console

from union import Resources
from union.cli._common import CommandBase
from union.remote import HuggingFaceModelInfo, ShardConfig, UnionRemote, VLLMShardArgs

DEFAULT_UNION_API_KEY = "UNION_API_KEY"
HF_TOKEN_KEY = "HF_TOKEN"


@click.group()
def cache():
    """Cache certain artifacts from remote registries."""


@cache.command(cls=CommandBase)
@click.argument("repo", type=str)
@click.option(
    "--artifact-name",
    type=str,
    required=False,
    default=None,
    help=(
        "Artifact name to use for the cached model. Must only contain alphanumeric characters, underscores, and "
        "hyphens. If not provided, the repo name will be used (replacing '.' with '-')."
    ),
)
@click.option(
    "--architecture",
    type=str,
    help="Model architecture, as given in HuggingFace config.json, For non transformer models use XGBoost, Custom etc.",
)
@click.option(
    "--task",
    default="auto",
    type=str,
    help="Model task, E.g, `generate`, `classify`, `embed`, `score` etc refer to VLLM docs, "
    "`auto` will try to discover this automatically",
)
@click.option(
    "--modality",
    type=str,
    multiple=True,
    help="Modalities supported by Model, E.g, `text`, `image`, `audio`, `video` etc refer to VLLM Docs",
)
@click.option("--format", type=str, help="Model serialization format, e.g safetensors, onnx, torchscript, joblib, etc")
@click.option(
    "--model-type",
    type=str,
    help="Model type, e.g, `transformer`, `xgboost`, `custom` etc. Model Type is important for non-transformer models."
    "For huggingface models, this is auto determined from config.json['model_type']",
)
@click.option("--short-description", type=str, help="Short description of the model")
@click.option("--force", type=int, help="Force caching of the model, pass --force=1/2/3... to force cache")
@click.option("--wait", is_flag=True, help="Wait for the model to be cached.")
@click.option("--hf-token-key", type=str, help="Union secret key with hugging face token", default=HF_TOKEN_KEY)
@click.option(
    "--union-api-key", type=str, help="Union secret key with admin permissions", default=DEFAULT_UNION_API_KEY
)
@click.option(
    "--cpu",
    type=str,
    help="Amount of CPU to use for downloading, (optionally) sharding, and caching hugging face model",
)
@click.option(
    "--gpu",
    type=str,
    help="Amount of GPU to use for downloading (optionally) sharding, and caching hugging face model",
)
@click.option(
    "--mem",
    type=str,
    help="Amount of Memory to use for downloading, (optionally) sharding, and caching hugging face model",
)
@click.option(
    "--ephemeral-storage",
    type=str,
    help="Amount of Ephemeral Storage to use for downloading, (optionally) sharding, and caching hugging face model",
)
@click.option(
    "--accelerator",
    default=None,
    type=click.Choice(
        [
            "nvidia-l4",
            "nvidia-l4-vws",
            "nvidia-l40s",
            "nvidia-a100",
            "nvidia-a100-80gb",
            "nvidia-a10g",
            "nvidia-tesla-k80",
            "nvidia-tesla-m60",
            "nvidia-tesla-p4",
            "nvidia-tesla-p100",
            "nvidia-tesla-t4",
            "nvidia-tesla-v100",
        ]
    ),
    help="The accelerator to use for downloading, (optionally) sharding, and caching hugging face model",
)
@click.option(
    "--shard-config",
    type=Path,
    help="The engine to shard the model with. Supported values are {{vllm}}",
)
def model_from_hf(
    repo: str,
    artifact_name: str,
    project: str,
    domain: str,
    architecture: str,
    task: str,
    modality: List[str],
    format: str,
    short_description: str,
    model_type: str,
    wait: bool,
    force: int,
    hf_token_key: str,
    union_api_key: str,
    cpu: str,
    gpu: str,
    mem: str,
    ephemeral_storage: str,
    accelerator: str,
    shard_config: Path,
):
    """Create a model with NAME."""
    remote = UnionRemote(default_domain=domain, default_project=project)

    if artifact_name is not None:
        assert re.match(r"^[a-zA-Z0-9_-]+$", artifact_name), (
            "Artifact name must only contain alphanumeric characters, underscores, and hyphens"
        )

    if shard_config is not None:
        with shard_config.open() as f:
            shard_config_dict = yaml.safe_load(f)
            shard_config = ShardConfig(
                engine=shard_config_dict["engine"],
                args=VLLMShardArgs(**shard_config_dict["args"]),
            )

    info = HuggingFaceModelInfo(
        repo=repo,
        artifact_name=artifact_name,
        architecture=architecture,
        task=task,
        modality=modality,
        serial_format=format,
        model_type=model_type,
        short_description=short_description,
        shard_config=shard_config,
    )
    cache_exec = remote._create_model_from_hf(
        info=info,
        hf_token_key=hf_token_key,
        union_api_key=union_api_key,
        retry=force,
        resources=Resources(cpu=cpu, mem=mem, gpu=gpu, ephemeral_storage=ephemeral_storage),
        accelerator=accelerator,
    )
    c = Console()
    url = cache_exec.execution_url
    c.print(
        f"🔄 Started background process to cache model from Hugging Face repo {repo}.\n"
        f" Check the console for status at [link={url}]{url}[/link]"
    )
    if wait:
        with c.status("Waiting for model to be cached...", spinner="dots"):
            cache_exec = cache_exec.wait(poll_interval=2)

        model_uri = cache_exec.outputs["artifact"].model_uri
        c.print(f"Cached model at: [cyan]{cache_exec.outputs['artifact'].blob}[/cyan]")
        c.print(f"Model Artifact ID: [green]{model_uri}[/green]")
        c.print()
        c.print("To deploy this model run:")
        c.print(f"union deploy model --project {project} --domain {domain} {model_uri}")



import warnings
from typing import TYPE_CHECKING, Any, Dict, Optional, Union

from flytekit.core.array_node_map_task import ArrayNodeMapTask
from flytekit.core.launch_plan import LaunchPlan
from flytekit.core.python_function_task import PythonFunctionTask
from flytekit.core.task import ReferenceTask

from union.map._union_array_node import UnionArrayNode

if TYPE_CHECKING:
    from flytekit.remote import FlyteLaunchPlan


def map(
    target: Union[LaunchPlan, PythonFunctionTask, "FlyteLaunchPlan"],
    bound_inputs: Optional[Dict[str, Any]] = None,
    concurrency: Optional[int] = None,
    min_successes: Optional[int] = None,
    min_success_ratio: float = 1.0,
    **kwargs,
):
    """
    Use to map over tasks, actors, launch plans, reference tasks and launch plans, and remote tasks and
    launch plans.

    :param target: The Flyte entity of which will be mapped over
    :param bound_inputs: Inputs that are bound to the array node and will not be mapped over
    :param concurrency: If specified, this limits the number of mapped tasks than can run in parallel to the given batch
        size. If the size of the input exceeds the concurrency value, then multiple batches will be run serially until
        all inputs are processed. If set to 0, this means unbounded concurrency. If left unspecified, this means the
        array node will inherit parallelism from the workflow
    :param min_successes: The minimum number of successful executions
    :param min_success_ratio: The minimum ratio of successful executions
    """
    from flytekit.remote import FlyteLaunchPlan

    if isinstance(target, (LaunchPlan, FlyteLaunchPlan, ReferenceTask)):
        if isinstance(target, ReferenceTask):
            if bound_inputs:
                warnings.warn("bound_inputs are not supported for tasks yet")
        return UnionArrayNode(
            target=target,
            bound_inputs=bound_inputs,
            concurrency=concurrency,
            min_successes=min_successes,
            min_success_ratio=min_success_ratio,
        )
    return ArrayNodeMapTask(
        target,
        bound_inputs_values=bound_inputs,
        concurrency=concurrency,
        min_success_ratio=min_success_ratio,
        min_successes=min_successes,
        **kwargs,
    )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hooks/payload.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from flyteidl.event import cloudevents_pb2 as flyteidl_dot_event_dot_cloudevents__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13hooks/payload.proto\x12\x0e\x63loudidl.hooks\x1a flyteidl/event/cloudevents.proto\"J\n\x0c\x45ventFilters\x12:\n\x0b\x65vent_types\x18\x01 \x03(\x0e\x32\x19.cloudidl.hooks.EventTypeR\neventTypes\"M\n\x13StreamEventsRequest\x12\x36\n\x07\x66ilters\x18\x01 \x01(\x0b\x32\x1c.cloudidl.hooks.EventFiltersR\x07\x66ilters\"\xb1\x02\n\x14StreamEventsResponse\x12\\\n\x12workflow_execution\x18\x01 \x01(\x0b\x32+.flyteidl.event.CloudEventWorkflowExecutionH\x00R\x11workflowExecution\x12P\n\x0enode_execution\x18\x02 \x01(\x0b\x32\'.flyteidl.event.CloudEventNodeExecutionH\x00R\rnodeExecution\x12P\n\x0etask_execution\x18\x03 \x01(\x0b\x32\'.flyteidl.event.CloudEventTaskExecutionH\x00R\rtaskExecution\x12\x0e\n\x02id\x18\x04 \x01(\tR\x02idB\x07\n\x05\x65vent\")\n\x17\x41\x63knowledgeEventRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"\x1a\n\x18\x41\x63knowledgeEventResponse*\\\n\tEventType\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x16\n\x12WORKFLOW_EXECUTION\x10\x01\x12\x12\n\x0eNODE_EXECUTION\x10\x02\x12\x12\n\x0eTASK_EXECUTION\x10\x03\x42\xa7\x01\n\x12\x63om.cloudidl.hooksB\x0cPayloadProtoH\x02P\x01Z(github.com/unionai/cloud/gen/pb-go/hooks\xa2\x02\x03\x43HX\xaa\x02\x0e\x43loudidl.Hooks\xca\x02\x0e\x43loudidl\\Hooks\xe2\x02\x1a\x43loudidl\\Hooks\\GPBMetadata\xea\x02\x0f\x43loudidl::Hooksb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'hooks.payload_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\022com.cloudidl.hooksB\014PayloadProtoH\002P\001Z(github.com/unionai/cloud/gen/pb-go/hooks\242\002\003CHX\252\002\016Cloudidl.Hooks\312\002\016Cloudidl\\Hooks\342\002\032Cloudidl\\Hooks\\GPBMetadata\352\002\017Cloudidl::Hooks'
  _globals['_EVENTTYPE']._serialized_start=607
  _globals['_EVENTTYPE']._serialized_end=699
  _globals['_EVENTFILTERS']._serialized_start=73
  _globals['_EVENTFILTERS']._serialized_end=147
  _globals['_STREAMEVENTSREQUEST']._serialized_start=149
  _globals['_STREAMEVENTSREQUEST']._serialized_end=226
  _globals['_STREAMEVENTSRESPONSE']._serialized_start=229
  _globals['_STREAMEVENTSRESPONSE']._serialized_end=534
  _globals['_ACKNOWLEDGEEVENTREQUEST']._serialized_start=536
  _globals['_ACKNOWLEDGEEVENTREQUEST']._serialized_end=577
  _globals['_ACKNOWLEDGEEVENTRESPONSE']._serialized_start=579
  _globals['_ACKNOWLEDGEEVENTRESPONSE']._serialized_end=605
# @@protoc_insertion_point(module_scope)



"""All imports to union in this file should be in the function definition.

This plugin is loaded by flytekit, so any imports to union can lead to circular imports.
"""

from contextlib import suppress
from typing import TYPE_CHECKING, List, Optional

import click
from click import Group, Option
from flytekit import CachePolicy

if TYPE_CHECKING:
    from union.remote import UnionRemote

from union.cache import CacheFunctionBody
from union.cli._cache import cache


class UnionAIPlugin:
    @staticmethod
    def get_remote(
        config: Optional[str],
        project: str,
        domain: str,
        data_upload_location: Optional[str] = None,
    ) -> "UnionRemote":
        from union.remote import UnionRemote

        return UnionRemote(
            config=UnionAIPlugin._get_config_for_remote(config),
            default_project=project,
            default_domain=domain,
            data_upload_location=data_upload_location,
        )

    @staticmethod
    def _get_config_for_remote(config: Optional[str], default_to_union_semantics: bool = False):
        """Get config when creating a UnionRemote when using the CLI."""
        from union._config import _UNION_CONFIG, ConfigSource, _get_config_obj

        if _UNION_CONFIG.config_source == ConfigSource.CLI.value:
            # The config that is passed through the CLI with `--config` will be handled by
            # `UnionRemote` itself
            config = None
        elif not _UNION_CONFIG.is_direct_union_cli_call:
            # Use pyflyte's default config when not using `union` directly
            config = _get_config_obj(config, default_to_union_semantics=default_to_union_semantics)
        return config

    @staticmethod
    def configure_pyflyte_cli(main: Group) -> Group:
        """Configure pyflyte's CLI."""

        import union._config

        # Import filesystems here so they get copied over properly when union is installed in
        # development mode
        import union.filesystems
        from union._config import _get_config_obj, _get_image_builder_priority
        from union.cli._create import create
        from union.cli._delete import delete
        from union.cli._deploy import deploy
        from union.cli._get import get
        from union.cli._info import info
        from union.cli._start import start
        from union.cli._stop import stop
        from union.cli._update import update
        from union.ucimage._image_builder import _register_union_image_builder

        def _cli_main_config_callback(ctx, param, value):
            from union._config import ConfigSource, ConfigWithSource
            from union._exceptions import UnionRequireConfigException

            # Set org based on config from `pyflyte --config` for downstream cli
            # commands to use.
            # Only register image builder for serverless
            with suppress(UnionRequireConfigException):
                if isinstance(value, str):
                    config_source = ConfigWithSource(config=value, source=ConfigSource.CLI)
                    config_obj = _get_config_obj(config_source)
                else:
                    config_obj = _get_config_obj(value)

                _register_union_image_builder(_get_image_builder_priority(config_obj.platform.endpoint))

            return value

        for p in main.params:
            if p.name == "config":
                p.callback = _cli_main_config_callback

        # Configure org at the top level:
        def _set_org(ctx, param, value):
            if value is not None:
                union._config._UNION_CONFIG.org = value

        main.params.append(
            click.Option(
                ["--org"],
                help="Set organization",
                hidden=True,
                callback=_set_org,
                expose_value=False,
            )
        )

        def update_or_create_group(new_group):
            try:
                main_group = main.commands[new_group.name]
                for name, command in new_group.commands.items():
                    main_group.add_command(command, name)
            except KeyError:
                main.add_command(new_group)

        new_groups = [create, delete, update, get, deploy, start, stop, cache]

        for group in new_groups:
            update_or_create_group(group)

        new_commands = [info]
        for command in new_commands:
            main.add_command(command)

        def yield_all_options():
            commands = [main]

            while commands:
                command = commands.pop()
                yield from (p for p in command.params if isinstance(p, Option))

                if isinstance(command, Group):
                    commands.extend(list(command.commands.values()))

        for param in yield_all_options():
            if param.name == "image_config" and param.default is not None and not callable(param.default):
                param.default = lambda: [union._config._get_default_image()]
            elif param.name == "project" and param.default is not None and not callable(param.default):
                param.default = union._config._GetDefaultProject(previous_default=param.default)

        return main

    @staticmethod
    def secret_requires_group() -> bool:
        """Return True if secrets require group entry."""
        return False

    @staticmethod
    def get_default_image() -> Optional[str]:
        """Return default image."""

        from union._config import _get_default_image

        return _get_default_image()

    @staticmethod
    def get_auth_success_html(endpoint: str) -> Optional[str]:
        """Get default success html. Return None to use flytekit's default success html."""
        from union._config import _get_auth_success_html

        return _get_auth_success_html(endpoint)

    @staticmethod
    def get_default_cache_policies() -> List[CachePolicy]:
        """Get default cache policies for tasks."""
        return [CacheFunctionBody()]



import copy
import os
import re
import shlex
import warnings
from contextlib import nullcontext
from copy import deepcopy
from dataclasses import dataclass, field
from datetime import timedelta
from enum import Enum
from inspect import getmembers
from pathlib import Path
from types import ModuleType
from typing import Any, Dict, List, Optional, Protocol, Union, runtime_checkable
from typing import Literal as L

from flyteidl.core.literals_pb2 import KeyValuePair
from flyteidl.core.tasks_pb2 import Container, ContainerPort, ExtendedResources, K8sObjectMetadata, K8sPod
from flyteidl.core.tasks_pb2 import Resources as ResourcesIDL
from flytekit import FlyteContextManager, ImageSpec, Resources
from flytekit.core.artifact import ArtifactQuery
from flytekit.core.pod_template import PodTemplate
from flytekit.core.resources import construct_extended_resources
from flytekit.extras.accelerators import BaseAccelerator
from flytekit.models.security import Secret
from mashumaro.codecs.json import JSONEncoder

from union.app._common import (
    INTERNAL_APP_ENDPOINT_PATTERN_ENV_VAR,
    _extract_files_loaded_from_cwd,
    patch_get_flytekit_and_union_for_pypi,
)
from union.app._frameworks import _is_fastapi_app
from union.internal.app.app_definition_pb2 import App as AppIDL
from union.internal.app.app_definition_pb2 import (
    AutoscalingConfig,
    Identifier,
    IngressConfig,
    Meta,
    Replicas,
    SecurityContext,
    Spec,
)
from union.internal.app.app_definition_pb2 import Concurrency as ConcurrencyIDL
from union.internal.app.app_definition_pb2 import Link as LinkIDL
from union.internal.app.app_definition_pb2 import RequestRate as RequestRateIDL
from union.internal.app.app_definition_pb2 import ScalingMetric as ScalingMetricIDL
from union.ucimage._image_builder import get_image_name, is_union_image


@dataclass
class AppRegistry:
    """Keeps track of all user defined applications"""

    apps: Dict[str, "App"] = field(default_factory=dict)

    def add_app(self, app: "App"):
        execution_state = FlyteContextManager.current_context().execution_state
        if execution_state.mode is None and app.name in self.apps:
            # This is running in a non execute context, so we check if the app name is already used:
            msg = f"App with name: {app.name} was already used. The latest definition will be used"
            warnings.warn(msg, UserWarning, stacklevel=2)

        self.apps[app.name] = app


APP_REGISTRY = AppRegistry()
# TODO: Add more protocols here when we deploy to another cloud providers
SUPPORTED_FS_PROTOCOLS = ["s3://", "gs://", "union://", "ufs://", "unionmeta://", "ums://", "abfs://"]
INVALID_APP_PORTS = [8012, 8022, 8112, 9090, 9091]
_PACKAGE_NAME_RE = re.compile(r"^[\w-]+")
_UNION_RUNTIME_NAME = "union-runtime"


def _is_union_runtime(package: str) -> bool:
    """Return True if `package` is union-runtime."""
    m = _PACKAGE_NAME_RE.match(package)
    if not m:
        return False
    name = m.group()
    return name == _UNION_RUNTIME_NAME


def _convert_union_runtime_to_serverless(packages: Optional[list[str]]) -> Optional[list[str]]:
    """Convert packages using union-runtime to union-runtime[serverless]"""
    if packages is None:
        return None

    union_runtime_length = len(_UNION_RUNTIME_NAME)
    new_packages = []
    for package in packages:
        if _is_union_runtime(package):
            version_spec = package[union_runtime_length:]
            new_packages.append(f"union-runtime[serverless]{version_spec}")
        else:
            new_packages.append(package)
    return new_packages


@dataclass
class URLQuery:
    name: str
    public: bool = False


ENV_NAME_RE = re.compile("^[_a-zA-Z][_a-zA-Z0-9]*$")
APP_NAME_RE = re.compile(r"[a-z0-9]([-a-z0-9]*[a-z0-9])?(\\.[a-z0-9]([-a-z0-9]*[a-z0-9])?)*")


def _has_file_extension(file_path) -> bool:
    _, ext = os.path.splitext(file_path)
    return bool(ext)


@dataclass
class Input:
    """
    Input for application.

    :param name: Name of input.
    :param value: Value for input.
    :param env_var: Environment name to set the value in the serving environment.
    :param download: When True, the input will be automatically downloaded. This
        only works if the value refers to an item in a object store. i.e. `s3://...`
    :param mount: If `value` is a directory, then the directory will be available
        at `mount`. If `value` is a file, then the file will be downloaded into the
        `mount` directory.
    :param ignore_patterns: If `value` is a directory, then this is a list of glob
        patterns to ignore.
    """

    class Type(Enum):
        File = "file"
        Directory = "directory"
        String = "string"
        _ArtifactUri = "artifact-uri"
        _UrlQuery = "url_query"  # Private type, users should not need this

    value: Union[str, ArtifactQuery, URLQuery]
    name: Optional[str] = None
    env_var: Optional[str] = None
    type: Optional[Type] = None
    download: bool = False
    mount: Optional[str] = None
    ignore_patterns: list[str] = field(default_factory=list)

    def __post_init__(self):
        if self.env_var is not None and ENV_NAME_RE.match(self.env_var) is None:
            msg = f"env_var ({self.env_var}) is not a valid environment name for shells"
            raise ValueError(msg)

        if self.name is None:
            if isinstance(self.value, ArtifactQuery):
                self.name = self.value.name
            elif isinstance(self.value, URLQuery):
                self.name = self.value.name
            elif isinstance(self.value, str):
                self.name = self.value
            else:
                msg = "If name is not provided, then the Input value must be an ArtifactQuery, URLQuery, or str"
                raise ValueError(msg)


@dataclass
class MaterializedInput:
    value: str
    type: Optional[Input.Type] = None


@dataclass
class AppSerializationSettings:
    """Runtime settings for creating an AppIDL"""

    org: str
    project: str
    domain: str
    is_serverless: bool

    desired_state: Spec.DesiredState
    materialized_inputs: dict[str, MaterializedInput] = field(default_factory=dict)
    additional_distribution: Optional[str] = None


@dataclass
class InputBackend:
    """
    Input information for the backend.
    """

    name: str
    value: str
    download: bool
    type: Optional[Input.Type] = None
    env_var: Optional[str] = None
    dest: Optional[str] = None
    ignore_patterns: List[str] = field(default_factory=list)

    def __post_init__(self):
        if self.type is None:
            if any(self.value.startswith(proto) for proto in SUPPORTED_FS_PROTOCOLS):
                if _has_file_extension(self.value):
                    self.type = Input.Type.File
                else:
                    self.type = Input.Type.Directory
            else:
                self.type = Input.Type.String

        if self.type == Input.Type.String:
            # Nothing to download for a string
            self.download = False

        # If the type is a file or directory and there is a destination, then we
        # automatically assume it is going to be downloaded
        # TODO: In the future, we may mount this, so there is no need to download it
        # with the runtime.
        if self.type in (Input.Type.File, Input.Type.Directory) and self.dest is not None:
            self.download = True

    @classmethod
    def from_input(cls, user_input: Input, settings: AppSerializationSettings) -> "InputBackend":
        if isinstance(user_input.value, str) and user_input.type != Input.Type._ArtifactUri:
            value = user_input.value
            input_type = user_input.type
        else:
            # ArtifactQuery or URLQuery
            try:
                materialized_input = settings.materialized_inputs[user_input.name]
                value = materialized_input.value
                input_type = materialized_input.type or user_input.type
            except KeyError:
                msg = f"Did not materialize {user_input.name}"
                raise ValueError(msg)

        return InputBackend(
            name=user_input.name,
            value=value,
            download=user_input.download,
            env_var=user_input.env_var,
            type=input_type,
            dest=user_input.mount,
            ignore_patterns=user_input.ignore_patterns,
        )


@dataclass
class ServeConfig:
    """
    Configuration for serve runtime.

    :param code_uri: Location of user code in an object store (s3://...)
    :param user_inputs: User inputs. Passed in by `app.inputs`
    """

    code_uri: str  # location of user code
    inputs: List[InputBackend]


SERVE_CONFIG_ENCODER = JSONEncoder(ServeConfig)


@dataclass
class ResolvedInclude:
    src: str
    dest: str


class ScalingMetric:
    @dataclass(frozen=True)
    class Concurrency:
        """
        Use this to specify the concurrency metric for autoscaling, i.e. the number of concurrent requests at a replica
         at which to scale up.
        """

        val: int

        def __post_init__(self):
            if self.val < 1:
                raise ValueError("Concurrency must be greater than or equal to 1")

        def _to_union_idl(self) -> ConcurrencyIDL:
            return ConcurrencyIDL(target_value=self.val)

    @dataclass
    class RequestRate:
        """
        Use this to specify the request rate metric for autoscaling, i.e. the number of requests per second at a replica
         at which to scale up.
        """

        val: int

        def __post_init__(self):
            if self.val < 1:
                raise ValueError("Request rate must be greater than or equal to 1")

        def _to_union_idl(self) -> RequestRateIDL:
            return RequestRateIDL(target_value=self.val)

    @staticmethod
    def _to_union_idl(metric: Optional[Union[RequestRate, Concurrency]]) -> Optional[ScalingMetricIDL]:
        if metric is None:
            return None
        request_rate = None
        concurrency = None
        if isinstance(metric, ScalingMetric.RequestRate):
            request_rate = metric._to_union_idl()
        if isinstance(metric, ScalingMetric.Concurrency):
            concurrency = metric._to_union_idl()
        return ScalingMetricIDL(
            request_rate=request_rate,
            concurrency=concurrency,
        )

    @staticmethod
    def _validate(metric: Optional[Union[RequestRate, Concurrency]]):
        if metric is None:
            return
        if not isinstance(metric, (ScalingMetric.RequestRate, ScalingMetric.Concurrency)):
            raise ValueError("scaling_metric must be an instance of Concurrency or RequestRate")


@runtime_checkable
class AppConfigProtocol(Protocol):
    def before_to_union_idl(self, app: "App", settings: AppSerializationSettings):
        """Modify app in place at the beginning of `App._to_union_idl`."""


@dataclass
class Link:
    path: str
    title: str
    is_relative: bool = False


@dataclass
class App:
    """
    App specification.

    :param name: The name of the application.
    :param container_image: The container image to use for the application.
    :param args: Entrypoint to start application.
    :param command: Command to start application.
    :param port: Port application listens to. Currently, this must be 8080 and the application
        must listen on 8080.
    :param requests: Compute resource requests for application.
    :param secrets: Secrets that are requested for application.
    :param limits: Compute resource limits for application.
    :param include: Files to include for your application.
    :param inputs: Inputs for the application.
    :param env: Environment variables to set for the application.
    :param min_replicas: Minimum number of replicas (ignore if autoscaling is set).
    :param max_replicas: Maximum number of replicas (ignore if autoscaling is set).
    :param scaledown_after: Time to wait before scaling down a replica after it has been idle.
    :param scaling_metric: Autoscale based on a parameter, e.g. request rate or concurrency
      (others may be added in the future).
    :param cluster_pool: The target cluster_pool where the app should be deployed.
    :param requires_auth: Public URL does not require any authentication
    :param type: Type of app
    :param description: Description of app
    :param framework_app: Object for serving framework. When this is set, all user defined files are uploaded.
        - For FastAPI, args is set to `uvicorn module_name:app_name --port port`. For more control
          you can set `args` directly.
    :param dependencies: List of apps that this app depends on.
    :param subdomain: Custom subdomain for your app.
    :param custom_domain: Custom full domain for your app.
    :param links: Links to external URLs or relative paths.
    :param shared_memory: If True, then shared memory will be attached to the container where the size is equal
            to the allocated memory. If str, then the shared memory is set to that size.
    """

    @dataclass
    class Port:
        port: int
        name: Optional[str] = None

    name: str
    container_image: Union[str, ImageSpec, PodTemplate]
    port: Optional[Union[int, Port]] = None
    limits: Optional[Resources] = None
    requests: Optional[Resources] = None
    secrets: List[Secret] = field(default_factory=list)
    args: Optional[Union[List[str], str]] = None
    command: Optional[Union[List[str], str]] = None
    min_replicas: int = 0
    max_replicas: int = 1
    scaledown_after: Optional[Union[int, timedelta]] = None
    scaling_metric: Optional[Union[ScalingMetric.Concurrency, ScalingMetric.RequestRate]] = None
    include: List[str] = field(default_factory=list)
    inputs: List[Input] = field(default_factory=list)
    env: dict = field(default_factory=dict)
    cluster_pool: str = "default"
    accelerator: Optional[BaseAccelerator] = None
    requires_auth: bool = True
    type: Optional[str] = None
    description: Optional[str] = None
    framework_app: Optional[Any] = None
    dependencies: List["App"] = field(default_factory=list)
    config: Optional[AppConfigProtocol] = None
    subdomain: Optional[str] = None
    custom_domain: Optional[str] = None
    links: List[Link] = field(default_factory=list)
    shared_memory: Optional[Union[L[True], str]] = None

    _include_resolved: Optional[List[ResolvedInclude]] = field(default=None, init=False)
    _port: Optional[Port] = field(default=None, init=False)
    _framework_variable_name: Optional[str] = field(default=None, init=False)
    _module_name: Optional[str] = field(default=None, init=False)

    def _validate_autoscaling_config(self):
        if self.min_replicas < 0:
            raise ValueError("min_replicas must be greater than or equal to 0")
        if self.max_replicas < 1 or self.max_replicas < self.min_replicas:
            raise ValueError(f"max_replicas must be greater than or equal to 1 or min_replicas:{self.min_replicas}")
        if self.scaledown_after is not None:
            if isinstance(self.scaledown_after, int):
                self.scaledown_after = timedelta(seconds=self.scaledown_after)
            if not 6 <= self.scaledown_after.total_seconds() <= 3600:
                raise ValueError("scaledown_after must be between 6 seconds and 1 hour")
        ScalingMetric._validate(self.scaling_metric)

    def _validate_ports(self):
        if isinstance(self.port, int):
            self._port = App.Port(port=self.port)
        elif isinstance(self.port, App.Port):
            self._port = self.port
        else:
            msg = "Expected port to be int, list[int], App.Port, or list[App.Port]"
            raise ValueError(msg)

        if self._port.port in INVALID_APP_PORTS:
            invalid_ports = ", ".join(str(p) for p in INVALID_APP_PORTS)
            msg = f"port {self._port.port} is not allowed. Please do not use ports: {invalid_ports}"
            raise ValueError(msg)

    def __post_init__(self):
        self._validate_autoscaling_config()

        if self.inputs and self.command is not None:
            msg = "To use inputs, set command=None to use the union-serve entrypoint"
            raise ValueError(msg)

        if self.include and self.command is not None:
            msg = "To use include, set command=None to use th union-serve entrypoint"
            raise ValueError(msg)

        if not self.framework_app:
            if self.args is None and self.command is None:
                msg = "Provide args or command to start your app. If you have default entrypoint then set command=[]"
                raise ValueError(msg)

            if self.port is None:
                msg = "Provide a port to connect to"
                raise ValueError(msg)
        else:
            if self.command is not None:
                msg = "When using framework_app, set command=None to use the union-runtime endpoint"
                raise ValueError(msg)

            if (
                self.type is None
                and hasattr(self.framework_app, "__class__")
                and hasattr(self.framework_app.__class__, "__name__")
            ):
                self.type = self.framework_app.__class__.__name__

            if self.port is None:
                self.port = 8080

        self._validate_ports()

        match_re = APP_NAME_RE.fullmatch(self.name)
        if not match_re:
            raise ValueError("App name must consist of lower case alphanumeric characters, '-' or '.'")

        if self.limits is None and self.requests is None:
            msg = "Either limits or requests must be set"
            raise ValueError(msg)

        if self.limits and self.limits.ephemeral_storage is None:
            self.limits.ephemeral_storage = "20Gi"

        if self.requests and self.requests.ephemeral_storage is None:
            self.requests.ephemeral_storage = "10Gi"

        APP_REGISTRY.add_app(self)

    @property
    def include_resolved(self) -> List[ResolvedInclude]:
        # Already resolved
        if self._include_resolved is not None:
            return self._include_resolved

        # Not resolved yet, so we resolve based on the cwd. This only happens when using
        # AppRemote programmatically and not from `union deploy`
        self._resolve_include(Path.cwd(), Path.cwd())
        return self._include_resolved

    def _attach_registration_scope(self, module: Optional[ModuleType], module_name: Optional[str]) -> "App":
        """
        Attach variable name to the object
        """
        if self.framework_app:
            # extract variable name from module
            for var_name, obj in getmembers(module):
                if obj is self.framework_app:
                    self._framework_variable_name = var_name
                    break
            else:  # no break
                msg = "Unable to find framework_app in the scope of App"
                raise RuntimeError(msg)
        self._module_name = module_name
        return self

    def _resolve_include(self, app_directory: Path, cwd: Path) -> "App":
        """
        Resolve include based on the working_dir.

        If a path in `include` is prefixed with "./", then those files are
        assumed to be relative to the file that has the App object.
        """
        relative_prefix = "./"
        seen_dests = set()

        included_resolved = []

        all_includes = self.include

        if self.framework_app is not None:
            all_includes.extend(_extract_files_loaded_from_cwd(cwd))

        for file in all_includes:
            normed_file = os.path.normpath(file)
            if file.startswith(relative_prefix):
                # File is relative to the app_directory:
                src_dir = app_directory
            else:
                src_dir = cwd

            if "*" in normed_file:
                new_srcs = src_dir.rglob(normed_file)
            else:
                new_srcs = [src_dir / normed_file]

            for new_src in new_srcs:
                dest = new_src.relative_to(src_dir).as_posix()
                if dest in seen_dests:
                    msg = f"{dest} is in include multiple times. Please remove one of them."
                    raise ValueError(msg)

                seen_dests.add(dest)

                included_resolved.append(ResolvedInclude(src=os.fspath(new_src), dest=os.fspath(dest)))

        self._include_resolved = included_resolved
        return self

    def _get_image(self, container_image: Union[str, ImageSpec], settings: AppSerializationSettings) -> str:
        if isinstance(container_image, str):
            if settings.is_serverless and not is_union_image(container_image):
                # Serverless expects the image to be built by the serverless runtime.
                # If the image isn't a union image, then we need to build it.
                container_image = ImageSpec(name=get_image_name(container_image), base_image=container_image)
            else:
                return container_image

        image_spec = container_image

        if settings.is_serverless and image_spec.packages is not None:
            image_spec = deepcopy(image_spec)
            new_packages = _convert_union_runtime_to_serverless(image_spec.packages)
            image_spec.packages = new_packages

        return self._build_image(image_spec)

    def _build_image(self, image_spec: ImageSpec):
        from flytekit.image_spec.image_spec import ImageBuildEngine

        if self.framework_app is None:
            # framework apps are defined with union, so we need to install union
            # in the image builder.
            ctx = patch_get_flytekit_and_union_for_pypi()
        else:
            ctx = nullcontext()

        # TODO: Remove this patching
        # Do not install flytekit or union when building image
        with ctx:
            ImageBuildEngine.build(image_spec)
        return image_spec.image_name()

    def _construct_args_for_framework(self, framework_app: Any) -> Optional[str]:
        # Framework specific argument adjustments here
        if _is_fastapi_app(framework_app):
            if self._module_name is None or self._framework_variable_name is None:
                raise ValueError("Unable to find module name")
            return f"uvicorn {self._module_name}:{self._framework_variable_name} --port {self.port}"

        return None

    def _get_args(self) -> List[str]:
        args = self.args

        # Framework specific argument adjustments here
        if self.framework_app is not None and args is None:
            args = self._construct_args_for_framework(self.framework_app)

        if args is None:
            return []
        elif isinstance(args, str):
            return shlex.split(args)
        else:
            # args is a list
            return args

    def _get_command(self, settings: AppSerializationSettings) -> List[str]:
        if self.command is None:
            cmd = ["union-serve"]

            serve_config = ServeConfig(
                code_uri=settings.additional_distribution,
                inputs=[InputBackend.from_input(user_input, settings) for user_input in self.inputs],
            )

            cmd.extend(["--config", SERVE_CONFIG_ENCODER.encode(serve_config)])

            return [*cmd, "--"]
        elif isinstance(self.command, str):
            return shlex.split(self.command)
        else:
            # args is a list
            return self.command

    def _get_resources(self) -> ResourcesIDL:
        requests_idl = []
        limits_idl = []

        # Make sure both limits and requests are set. During __post_init__ we require
        # either limits or requests to be set
        limits = self.limits or self.requests
        requests = self.requests or self.limits

        if requests.cpu is not None:
            requests_idl.append(ResourcesIDL.ResourceEntry(name=ResourcesIDL.ResourceName.CPU, value=str(requests.cpu)))
        if requests.mem is not None:
            requests_idl.append(
                ResourcesIDL.ResourceEntry(name=ResourcesIDL.ResourceName.MEMORY, value=str(requests.mem))
            )
        if requests.gpu is not None:
            requests_idl.append(ResourcesIDL.ResourceEntry(name=ResourcesIDL.ResourceName.GPU, value=str(requests.gpu)))
        if requests.ephemeral_storage is not None:
            requests_idl.append(
                ResourcesIDL.ResourceEntry(
                    name=ResourcesIDL.ResourceName.EPHEMERAL_STORAGE, value=str(requests.ephemeral_storage)
                )
            )

        if limits.cpu is not None:
            limits_idl.append(ResourcesIDL.ResourceEntry(name=ResourcesIDL.ResourceName.CPU, value=str(limits.cpu)))
        if limits.mem is not None:
            limits_idl.append(ResourcesIDL.ResourceEntry(name=ResourcesIDL.ResourceName.MEMORY, value=str(limits.mem)))
        if limits.gpu is not None:
            limits_idl.append(ResourcesIDL.ResourceEntry(name=ResourcesIDL.ResourceName.GPU, value=str(limits.gpu)))
        if limits.ephemeral_storage is not None:
            limits_idl.append(
                ResourcesIDL.ResourceEntry(
                    name=ResourcesIDL.ResourceName.EPHEMERAL_STORAGE, value=str(limits.ephemeral_storage)
                )
            )

        return ResourcesIDL(requests=requests_idl, limits=limits_idl)

    def _get_container(self, settings: AppSerializationSettings) -> Container:
        container_ports = [ContainerPort(container_port=self._port.port, name=self._port.name)]

        return Container(
            image=self._get_image(self.container_image, settings),
            command=self._get_command(settings),
            args=self._get_args(),
            resources=self._get_resources(),
            ports=container_ports,
            env=[KeyValuePair(key=k, value=v) for k, v in self._get_env(settings).items()],
        )

    def _get_env(self, settings: AppSerializationSettings) -> dict:
        return self.env

    def _get_extended_resources(self) -> Optional[ExtendedResources]:
        return construct_extended_resources(accelerator=self.accelerator, shared_memory=self.shared_memory)

    def _to_union_idl(self, settings: AppSerializationSettings) -> AppIDL:
        if self.config is not None:
            self.config.before_to_union_idl(self, settings)

        security_context_kwargs = {}
        security_context = None
        if self.secrets:
            security_context_kwargs["secrets"] = [s.to_flyte_idl() for s in self.secrets]
        if not self.requires_auth:
            security_context_kwargs["allow_anonymous"] = True

        if security_context_kwargs:
            security_context = SecurityContext(**security_context_kwargs)

        scaling_metric = ScalingMetric._to_union_idl(self.scaling_metric)

        dur = None
        if self.scaledown_after:
            from google.protobuf.duration_pb2 import Duration

            dur = Duration()
            dur.FromTimedelta(self.scaledown_after)

        autoscaling = AutoscalingConfig(
            replicas=Replicas(min=self.min_replicas, max=self.max_replicas),
            scaledown_period=dur,
            scaling_metric=scaling_metric,
        )

        spec_kwargs = {}
        if isinstance(self.container_image, (str, ImageSpec)):
            spec_kwargs["container"] = self._get_container(settings)
        elif isinstance(self.container_image, PodTemplate):
            spec_kwargs["pod"] = self._get_k8s_pod(self.container_image, settings)
        else:
            msg = "container_image must be a str, ImageSpec or PodTemplate"
            raise ValueError(msg)

        from union.internal.app.app_definition_pb2 import Profile

        return AppIDL(
            metadata=Meta(
                id=Identifier(
                    org=settings.org,
                    project=settings.project,
                    domain=settings.domain,
                    name=self.name,
                ),
            ),
            spec=Spec(
                desired_state=settings.desired_state,
                ingress=IngressConfig(
                    private=False,
                    subdomain=self.subdomain if self.subdomain else None,
                    cname=self.custom_domain if self.custom_domain else None,
                ),
                autoscaling=autoscaling,
                security_context=security_context,
                cluster_pool=self.cluster_pool,
                extended_resources=self._get_extended_resources(),
                profile=Profile(
                    type=self.type,
                    short_description=self.description,
                ),
                links=[LinkIDL(path=link.path, title=link.title, is_relative=link.is_relative) for link in self.links]
                if self.links
                else None,
                **spec_kwargs,
            ),
        )

    @classmethod
    def _update_app_idl(cls, old_app_idl: AppIDL, new_app_idl: AppIDL) -> AppIDL:
        # Replace all lists with empty so that MergeFrom works out of the box.
        app_idl_ = AppIDL(
            metadata=old_app_idl.metadata,
            spec=new_app_idl.spec,
            status=old_app_idl.status,
        )
        # Make sure values set by the server and not by the app configuration is
        # preserved.
        if old_app_idl.spec.creator.ListFields():
            app_idl_.spec.creator.CopyFrom(old_app_idl.spec.creator)

        # Ingress subdomain could be configured by the server or overriden by the user
        app_idl_.spec.ingress.CopyFrom(old_app_idl.spec.ingress)
        app_idl_.spec.ingress.MergeFrom(new_app_idl.spec.ingress)
        return app_idl_

    def _get_k8s_pod(self, pod_template: PodTemplate, settings: AppSerializationSettings) -> K8sPod:
        """Convert pod_template into a K8sPod IDL."""
        import json

        from google.protobuf.json_format import Parse
        from google.protobuf.struct_pb2 import Struct

        pod_spec_dict = self._serialized_pod_spec(pod_template, settings)
        pod_spec_idl = Parse(json.dumps(pod_spec_dict), Struct())

        metadata = K8sObjectMetadata(
            labels=pod_template.labels,
            annotations=pod_template.annotations,
        )
        return K8sPod(pod_spec=pod_spec_idl, metadata=metadata)

    @staticmethod
    def _sanitize_resource_name(resource: ResourcesIDL.ResourceEntry) -> str:
        return ResourcesIDL.ResourceName.Name(resource.name).lower().replace("_", "-")

    def _serialized_pod_spec(
        self,
        pod_template: PodTemplate,
        settings: AppSerializationSettings,
    ) -> dict:
        """Convert pod spec into a dict."""
        from kubernetes.client import ApiClient
        from kubernetes.client.models import V1Container, V1ContainerPort, V1EnvVar, V1ResourceRequirements

        pod_template = copy.deepcopy(pod_template)

        if pod_template.pod_spec is None:
            return {}

        if pod_template.primary_container_name != "app":
            msg = "Primary container name must be 'app'"
            raise ValueError(msg)

        containers: list[V1Container] = pod_template.pod_spec.containers
        primary_exists = any(container.name == pod_template.primary_container_name for container in containers)

        if not primary_exists:
            msg = "Primary container does not exist with name 'app' does not exist"
            raise ValueError(msg)

        final_containers = []

        # Move all resource names into containers
        for container in containers:
            container.image = self._get_image(container.image, settings)

            if container.name == pod_template.primary_container_name:
                container.args = self._get_args()
                container.command = self._get_command(settings)

                limits, requests = {}, {}
                resources = self._get_resources()
                for resource in resources.limits:
                    limits[self._sanitize_resource_name(resource)] = resource.value
                for resource in resources.requests:
                    requests[self._sanitize_resource_name(resource)] = resource.value

                resource_requirements = V1ResourceRequirements(limits=limits, requests=requests)

                if limits or requests:
                    container.resources = resource_requirements

                if self.env:
                    container.env = [V1EnvVar(name=k, value=v) for k, v in self.env.items()] + (container.env or [])

                container.ports = [V1ContainerPort(container_port=self._port.port, name=self._port.name)] + (
                    container.ports or []
                )

            final_containers.append(container)

        pod_template.pod_spec.containers = final_containers
        return ApiClient().sanitize_for_serialization(pod_template.pod_spec)

    def query_endpoint(self, *, public: bool = False) -> URLQuery:
        """
        Query for endpoint.

        :param public: Whether to return the public or internal endpoint.
        :returns: Object representing a URL query.
        """
        return URLQuery(name=self.name, public=public)

    @property
    def endpoint(self) -> str:
        """
        Return endpoint for App.
        """
        pattern = os.getenv(INTERNAL_APP_ENDPOINT_PATTERN_ENV_VAR)
        if pattern is not None:
            # If the pattern exist, we are on remote.
            # NOTE: Should check that the app is in the same namespace as the current app.
            return pattern.replace("{app_fqdn}", self.name)

        from union.remote import UnionRemote

        remote = UnionRemote()
        app_remote = remote._app_remote
        app_idl = app_remote.get(name=self.name)
        url = app_idl.status.ingress.public_url
        return url


@dataclass
class FlyteConnectorApp(App):
    """
    FlyteConnector application specification that inherits from App.
    """

    # We can't expose prometheus port since knative's admission webhook doesn't support more than one container port.
    port: Union[int, App.Port] = field(default_factory=lambda: App.Port(port=8080, name="h2c"))
    type: str = "connector"
    min_replicas: int = 1  # We need at least 1 replica to spin up the connector app

    def __post_init__(self):
        # Override any user-provided args with required connector args
        base_args = ["pyflyte", "serve", "connector", "--port", "8080", "--prometheus_port", "9092"]

        # Add modules args if includes exist
        if self.include:
            # Convert file paths to module names
            modules = []
            for file_path in self.include:
                # Remove .py extension if exists
                module_name = os.path.splitext(os.path.basename(file_path))[0]
                modules.append(module_name)

            base_args.extend(["--modules", *modules])

        self.args = base_args
        self.command = None  # Ensure command is None since we're enforcing specific args
        # Call parent's post init for other validations
        super().__post_init__()



import asyncio
import dataclasses
import io
import json
import logging
import pathlib
import struct
import tempfile
import time
from collections import defaultdict
from typing import Any, Hashable, Protocol
from urllib.parse import urlparse

import aiofiles
import aiofiles.os
import numpy as np
import obstore
import pydantic
import torch
from typing_extensions import Annotated

from .config import (
    CHUNK_SIZE,
    MAX_CONCURRENCY,
)

logger = logging.getLogger(__name__)

LITTLE_ENDIAN_LONG_LONG_STRUCT_FORMAT = "<Q"

SAFETENSORS_FORMAT_KEY = "format"
SAFETENSORS_FORMAT_VALUE = "pt"
SAFETENSORS_SUFFIX = ".safetensors"
SAFETENSORS_DEFAULT_PATTERN = f"*{SAFETENSORS_SUFFIX}"
SAFETENSORS_SHARDED_PATTERN = f"model-rank-{{rank}}-part-*{SAFETENSORS_SUFFIX}"
SAFETENSORS_INTERNAL_METADATA_KEY = "__metadata__"
SAFETENSORS_INDEX_PATH = "model.safetensors.index.json"
SAFETENSORS_HEADER_BUFFER_SIZE = 8
SAFETENSORS_TO_TORCH_DTYPE = {
    "F64": torch.float64,
    "F32": torch.float32,
    "F16": torch.float16,
    "BF16": torch.bfloat16,
    "I64": torch.int64,
    "I32": torch.int32,
    "I16": torch.int16,
    "I8": torch.int8,
    "U8": torch.uint8,
    "BOOL": torch.bool,
    "F8_E5M2": torch.float8_e5m2,
    "F8_E4M3": torch.float8_e4m3fn,
}


class DownloadQueueEmpty(RuntimeError):
    pass


def prefetch(remote_model_path, local_model_path, exclude_safetensors=True):
    logger.info(f"Pre-fetching model artifacts from {remote_model_path} to {local_model_path}...")
    if exclude_safetensors:
        logger.info(f"Deferring download of safetensor files from {remote_model_path}")
    start = time.perf_counter()
    parsed_url = urlparse(remote_model_path)
    src_prefix = pathlib.Path(parsed_url.path.lstrip("/"))
    target_prefix = pathlib.Path(local_model_path)
    store = obstore_from_url(remote_model_path)
    reader = ObstoreParallelReader(store)
    try:
        asyncio.run(
            reader.download_files(
                src_prefix,
                target_prefix,
                exclude=[SAFETENSORS_DEFAULT_PATTERN] if exclude_safetensors else None,
            )
        )
    except* DownloadQueueEmpty:
        logger.warning("No model artifacts found to pre-fetch.")
    else:
        logger.info(f"Pre-fetched model artifacts in {time.perf_counter() - start:.2f}s")


def obstore_from_url(url, **kwargs):
    for maybe_store in (
        obstore.store.S3Store,
        obstore.store.GCSStore,
        obstore.store.AzureStore,
    ):
        try:
            return maybe_store.from_url(url, **kwargs)
        except obstore.exceptions.ObstoreError:
            pass
    raise ValueError(f"Could not find valid store for URL: {url}. Must be an S3, GCS, or Azure URI")


def prefix_exists(url: str) -> bool:
    store = obstore_from_url(url)
    prefix = urlparse(url).path.lstrip("/")
    for _ in obstore.list(store, prefix, chunk_size=1):
        return True
    return False


class BufferProtocol(Protocol):
    async def write(self, offset, length, value) -> None: ...

    async def read(self) -> memoryview: ...

    @property
    def complete(self) -> bool: ...


@dataclasses.dataclass
class _MemoryBuffer:
    arr: np.ndarray
    pending: int
    _closed: bool = False

    async def write(self, offset, length, value) -> None:
        self.arr[offset : offset + length] = value
        self.pending -= length

    async def read(self) -> memoryview:
        return memoryview(self.arr)

    @property
    def complete(self) -> bool:
        return self.pending == 0

    @classmethod
    def new(cls, size):
        return cls(arr=np.empty(size, dtype=np.uint8), pending=size)


@dataclasses.dataclass
class _FileBuffer:
    path: pathlib.Path
    pending: int
    _handle: io.FileIO | None = None
    _closed: bool = False

    async def write(self, offset, length, value) -> None:
        async with aiofiles.open(self.path, mode="r+b") as f:
            await f.seek(offset)
            await f.write(value)
        self.pending -= length

    async def read(self) -> memoryview:
        async with aiofiles.open(self.path, mode="rb") as f:
            return memoryview(await f.read())

    @property
    def complete(self) -> bool:
        return self.pending == 0

    @classmethod
    def new(cls, path, size):
        path.parent.mkdir(parents=True, exist_ok=True)
        path.touch()
        return cls(path=path, pending=size)


@dataclasses.dataclass
class Chunk:
    offset: int
    length: int


@dataclasses.dataclass
class Source:
    id: Hashable
    path: pathlib.Path
    length: int
    offset: int = 0
    metadata: Any | None = None


@dataclasses.dataclass
class DownloadTask:
    source: Source
    chunk: Chunk
    target: pathlib.Path | None = None


class ObstoreParallelReader:
    def __init__(
        self,
        store,
        *,
        chunk_size=CHUNK_SIZE,
        max_concurrency=MAX_CONCURRENCY,
    ):
        self._store = store
        self._chunk_size = chunk_size
        self._max_concurrency = max_concurrency

    def _chunks(self, size):
        offsets = np.arange(0, size, self._chunk_size)
        lengths = np.minimum(self._chunk_size, size - offsets)
        return zip(offsets, lengths)

    async def _as_completed(self, gen, transformer=None):
        inq = asyncio.Queue(self._max_concurrency * 2)
        outq = asyncio.Queue()
        sentinel = object()
        done = asyncio.Event()

        active = {}

        async def _fill():
            try:
                counter = 0
                async for task in gen:
                    if task.source.id not in active:
                        active[task.source.id] = (
                            _FileBuffer.new(task.target, task.source.length)
                            if task.target is not None
                            else _MemoryBuffer.new(task.source.length)
                        )
                    await inq.put(task)
                    counter += 1
                await inq.put(sentinel)
                if counter == 0:
                    raise DownloadQueueEmpty
            except asyncio.CancelledError:
                pass

        async def _worker():
            try:
                while not done.is_set():
                    task = await inq.get()
                    if task is sentinel:
                        inq.put_nowait(sentinel)
                        break
                    chunk_source_offset = task.chunk.offset + task.source.offset
                    buf = active[task.source.id]
                    await buf.write(
                        task.chunk.offset,
                        task.chunk.length,
                        await obstore.get_range_async(
                            self._store,
                            str(task.source.path),
                            start=chunk_source_offset,
                            end=chunk_source_offset + task.chunk.length,
                        ),
                    )
                    if not buf.complete:
                        continue
                    if transformer is not None:
                        result = await transformer(task.source, buf)
                    elif task.target is not None:
                        result = task.target
                    else:
                        result = task.source
                    outq.put_nowait((task.source.id, result))
                    del active[task.source.id]
            except asyncio.CancelledError:
                pass
            finally:
                done.set()

        # Yield results as they are completed
        async with asyncio.TaskGroup() as tg:
            tg.create_task(_fill())
            for _ in range(self._max_concurrency):
                tg.create_task(_worker())
            while not done.is_set():
                yield await outq.get()

        # Drain the output queue
        try:
            while True:
                yield outq.get_nowait()
        except asyncio.QueueEmpty:
            pass

    async def download_files(self, src_prefix, target_prefix, *paths, include=None, exclude=None):
        def _keep(path):
            if include is not None and not any(path.match(i) for i in include):
                return False
            if exclude is not None and any(path.match(e) for e in exclude):
                return False
            return True

        async def _list_downloadable():
            if paths:
                for path_ in paths:
                    path = src_prefix / path_
                    if _keep(path):
                        yield await obstore.head_async(self._store, str(path))
                return

            list_result = await obstore.list_with_delimiter_async(self._store, prefix=str(src_prefix))
            for obj in list_result["objects"]:
                path = pathlib.Path(obj["path"])
                if _keep(path):
                    yield obj

        async def _gen(tmpdir):
            async for obj in _list_downloadable():
                path = pathlib.Path(obj["path"])
                size = obj["size"]
                source = Source(id=path, path=path, length=size)
                # Strip src_prefix from path for destination
                rel_path = path.relative_to(src_prefix)
                for offset, length in self._chunks(size):
                    yield DownloadTask(
                        source=source,
                        target=tmpdir / rel_path,
                        chunk=Chunk(offset, length),
                    )

        def _transform_decorator(tmpdir):
            async def _transformer(source: Source, buf: BufferProtocol) -> None:
                target = target_prefix / buf.path.relative_to(tmpdir)
                await aiofiles.os.makedirs(target.parent, exist_ok=True)
                return await aiofiles.os.replace(buf.path, target)

            return _transformer

        with tempfile.TemporaryDirectory() as tmpdir:
            async for _ in self._as_completed(_gen(tmpdir), transformer=_transform_decorator(tmpdir)):
                pass

    async def get_ranges(self, gen, transformer=None):
        async def _gen():
            async for source in gen:
                for offset, length in self._chunks(source.length):
                    yield DownloadTask(source=source, chunk=Chunk(offset, length))

        async for result in self._as_completed(_gen(), transformer=transformer):
            yield result


def _dtype_to_torch_dtype(dtype: str) -> torch.dtype:
    try:
        return SAFETENSORS_TO_TORCH_DTYPE[dtype]
    except KeyError:
        raise ValueError(f"Unsupported dtype: {dtype}")


class TensorMetadata(pydantic.BaseModel):
    model_config = pydantic.ConfigDict(arbitrary_types_allowed=True)

    name: str
    shape: list[int]
    dtype: Annotated[torch.dtype, pydantic.BeforeValidator(_dtype_to_torch_dtype)]
    data_offsets: tuple[int, int]

    @pydantic.computed_field
    @property
    def size(self) -> int:
        start, end = self.data_offsets
        return end - start

    @pydantic.computed_field
    @property
    def length(self) -> int:
        count = 1
        for dim in self.shape:
            count *= dim
        return count

    def __len__(self):
        return self.length


class SafeTensorsMetadata(pydantic.BaseModel):
    path: str
    data_start: int
    tensors: list[TensorMetadata]


class SafeTensorsStreamer:
    def __init__(
        self,
        remote_path,
        local_path,
        chunk_size=CHUNK_SIZE,
        max_concurrency=MAX_CONCURRENCY,
        rank=0,
        tensor_parallel_size=1,
        store_kwargs=None,
    ):
        self._store = obstore_from_url(remote_path, **(store_kwargs or {}))
        parsed_url = urlparse(remote_path)
        self._bucket = parsed_url.netloc
        self._prefix = pathlib.Path(parsed_url.path.lstrip("/"))
        self._local_path = pathlib.Path(local_path)
        self._reader = ObstoreParallelReader(self._store, chunk_size=chunk_size, max_concurrency=max_concurrency)
        self._rank = rank
        self._tensor_parallel_size = tensor_parallel_size

    async def _parse_safetensors_metadata(self, path):
        header_len = await obstore.get_range_async(self._store, str(path), start=0, end=SAFETENSORS_HEADER_BUFFER_SIZE)
        header_size = struct.unpack(
            LITTLE_ENDIAN_LONG_LONG_STRUCT_FORMAT,
            header_len,
        )[0]
        header_data = json.loads(
            (
                await obstore.get_range_async(
                    self._store,
                    str(path),
                    start=SAFETENSORS_HEADER_BUFFER_SIZE,
                    end=SAFETENSORS_HEADER_BUFFER_SIZE + header_size,
                )
            ).to_bytes()
        )
        if (
            format := header_data.pop(SAFETENSORS_INTERNAL_METADATA_KEY, {}).get(SAFETENSORS_FORMAT_KEY)
        ) and format != SAFETENSORS_FORMAT_VALUE:
            raise ValueError(f"Unsupported format: {format}")
        return SafeTensorsMetadata(
            path=str(path),
            data_start=SAFETENSORS_HEADER_BUFFER_SIZE + header_size,
            tensors=[TensorMetadata.model_validate({"name": k, **v}) for k, v in header_data.items()],
        )

    async def _list_safetensors_files_with_index(self):
        # Get index of expected tensors if it exists
        weight_map_resp = await obstore.get_async(self._store, str(self._prefix / SAFETENSORS_INDEX_PATH))
        weight_map_bytes = bytes(await weight_map_resp.bytes_async())
        tensor_to_path_map = json.loads(weight_map_bytes)["weight_map"]

        # Create index for path -> tensors
        index = defaultdict(set)
        for tensor, path in tensor_to_path_map.items():
            index[path].add(tensor)

        return index.items()

    async def _load_safetensors_metadata_from_index(self):
        for path, expected in await self._list_safetensors_files_with_index():
            stm = await self._parse_safetensors_metadata(self._prefix / path)
            # Keep only the tensors we expect (should already be deduplicated)
            keep = {tm.name: tm for tm in filter(lambda tm: tm.name in expected, stm.tensors)}
            # We have missing tensors at the path. Bail out!
            if missing := expected - keep.keys():
                raise ValueError(f"Missing {len(missing)} tensors at {path!r}: {' '.join(missing)}")
            stm.tensors = list(keep.values())
            yield stm

    async def _list_safetensors_files_with_pattern(self, pattern):
        paths = set()
        list_result = await obstore.list_with_delimiter_async(self._store, prefix=str(self._prefix))
        for obj in list_result["objects"]:
            path = pathlib.Path(obj["path"])
            if path.match(pattern):
                paths.add(path)
        if not paths:
            raise ValueError(f"No files found matching pattern: {pattern}")
        return paths

    async def _load_safetensors_metadata_with_pattern(self, pattern):
        seen = set()
        stms = await asyncio.gather(
            *(
                self._parse_safetensors_metadata(path)
                for path in await self._list_safetensors_files_with_pattern(pattern)
            )
        )
        for stm in stms:
            stm.tensors = list(
                filter(
                    lambda tm: tm.name not in seen and not seen.add(tm.name),
                    stm.tensors,
                )
            )
            yield stm

    async def _load_safetensors_metadata(self):
        # When using tensor parallelism, we can't rely on the index. Fallback to using a pattern.
        if self._tensor_parallel_size > 1:
            async for stm in self._load_safetensors_metadata_with_pattern(
                SAFETENSORS_SHARDED_PATTERN.format(rank=self._rank)
            ):
                yield stm
            return

        # No tensor parallelism. Try to use the index first, then fallback to a pattern.
        try:
            async for stm in self._load_safetensors_metadata_from_index():
                yield stm
        except (
            obstore.exceptions.NotFoundError,
            json.decoder.JSONDecodeError,
            FileNotFoundError,
            KeyError,
        ):
            async for stm in self._load_safetensors_metadata_with_pattern(SAFETENSORS_DEFAULT_PATTERN):
                yield stm

    async def _get_tensors_async(self):
        async def _to_tensor(source: Source, buf: BufferProtocol) -> torch.Tensor:
            return torch.frombuffer(
                await buf.read(),
                dtype=source.metadata.dtype,
                count=len(source.metadata),
                offset=0,
            ).view(source.metadata.shape)

        async def _gen():
            async for stm in self._load_safetensors_metadata():
                for tm in stm.tensors:
                    yield Source(
                        id=tm.name,
                        path=stm.path,
                        length=tm.size,
                        offset=stm.data_start + tm.data_offsets[0],
                        metadata=tm,
                    )

        # Yield tensors as they are downloaded
        async for name, tensor in self._reader.get_ranges(_gen(), transformer=_to_tensor):
            yield name, tensor

    def get_tensors(self):
        logger.info("Streaming tensors...")
        start = time.perf_counter()
        counter = 0
        gen = self._get_tensors_async()
        with asyncio.Runner() as runner:
            try:
                while True:
                    yield runner.run(gen.__anext__())
                    counter += 1
            except StopAsyncIteration:
                pass
        logger.info(f"Streamed {counter} tensors in {time.perf_counter() - start:.2f}s")



import logging
import os
from dataclasses import dataclass
from typing import TYPE_CHECKING

from ._models import AppConfigProtocol, AppSerializationSettings, Link

if TYPE_CHECKING:
    from ._models import App

logger = logging.getLogger(__name__)


@dataclass
class ArizeConfig(AppConfigProtocol):
    endpoint: str  # <host>/organizations/<org_id>/spaces/<space_id>/models[/<model_id>]

    def before_to_union_idl(self, app: "App", settings: AppSerializationSettings):
        """Modify app in place at the beginning of `App._to_union_idl`."""

        link = Link(path=self.endpoint, title="Arize Traces")
        app.links.append(link)


@dataclass
class PhoenixConfig(AppConfigProtocol):
    endpoint: str  # <host> or <host>/s/<space_id>
    project: str  # name of the project

    def before_to_union_idl(self, app: "App", settings: AppSerializationSettings):
        """Modify app in place at the beginning of `App._to_union_idl`."""
        app.env["PHOENIX_COLLECTOR_ENDPOINT"] = self.endpoint
        app.env["PHOENIX_PROJECT_NAME"] = self.project

        try:
            from phoenix.client import Client
        except ImportError:
            logger.warning(
                "Missing Phoenix client. "
                "You can install it with `pip install arize-phoenix-client`. Skipping link generation."
            )
            return

        api_key = os.environ.get("PHOENIX_API_KEY")
        if not api_key:
            logger.warning("PHOENIX_API_KEY not found locally. Skipping link generation.")
            return

        os.environ["PHOENIX_CLIENT_HEADERS"] = f"api_key={api_key}"

        try:
            client = Client(base_url=self.endpoint)

            # Get list of projects and map names to IDs
            projects = client.projects.list()
            project_name_to_id = {p["name"]: p["id"] for p in projects}

            # Check if the project exists
            if self.project in project_name_to_id:
                project_id = project_name_to_id[self.project]
            else:
                logger.warning(f"Project does not exist: {self.project}. Creating project.")
                client.projects.create(name=self.project)

                # Retrieve the project ID after creation
                project = client.projects.get(project_name=self.project)
                project_id = project.get("id")

                if not project_id:
                    logger.warning(f"Failed to retrieve project ID for: {self.project}")
                    return

            phoenix_link = f"{self.endpoint.rstrip('/')}/projects/{project_id}/spans"
            link = Link(path=phoenix_link, title="Phoenix Traces")
            app.links.append(link)

        except Exception as e:
            logger.warning(f"Failed to generate Phoenix link: {e}")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: common/role.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.common import authorization_pb2 as common_dot_authorization__pb2
from union.internal.common import identifier_pb2 as common_dot_identifier__pb2
from union.internal.validate.validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11\x63ommon/role.proto\x12\x0f\x63loudidl.common\x1a\x1a\x63ommon/authorization.proto\x1a\x17\x63ommon/identifier.proto\x1a\x17validate/validate.proto\"\xa7\x02\n\x04Role\x12\x39\n\x02id\x18\x01 \x01(\x0b\x32\x1f.cloudidl.common.RoleIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x02id\x12\x41\n\x0bpermissions\x18\x02 \x03(\x0b\x32\x1b.cloudidl.common.PermissionB\x02\x18\x01R\x0bpermissions\x12\x36\n\trole_spec\x18\x03 \x01(\x0b\x32\x19.cloudidl.common.RoleSpecR\x08roleSpec\x12\x36\n\trole_type\x18\x04 \x01(\x0e\x32\x19.cloudidl.common.RoleTypeR\x08roleType\x12\x31\n\x07\x61\x63tions\x18\x05 \x03(\x0e\x32\x17.cloudidl.common.ActionR\x07\x61\x63tions\",\n\x08RoleSpec\x12 \n\x0b\x64\x65scription\x18\x01 \x01(\tR\x0b\x64\x65scription*\x83\x02\n\x08RoleType\x12\x12\n\x0eROLE_TYPE_NONE\x10\x00\x12\x13\n\x0fROLE_TYPE_ADMIN\x10\x01\x12\x19\n\x15ROLE_TYPE_CONTRIBUTOR\x10\x02\x12\x14\n\x10ROLE_TYPE_VIEWER\x10\x03\x12\x14\n\x10ROLE_TYPE_CUSTOM\x10\x04\x12\x1d\n\x19ROLE_TYPE_CLUSTER_MANAGER\x10\x05\x12!\n\x1dROLE_TYPE_FLYTE_PROJECT_ADMIN\x10\x06\x12\x1f\n\x1bROLE_TYPE_SERVERLESS_VIEWER\x10\x07\x12$\n ROLE_TYPE_SERVERLESS_CONTRIBUTOR\x10\x08\x42\xaa\x01\n\x13\x63om.cloudidl.commonB\tRoleProtoH\x02P\x01Z)github.com/unionai/cloud/gen/pb-go/common\xa2\x02\x03\x43\x43X\xaa\x02\x0f\x43loudidl.Common\xca\x02\x0f\x43loudidl\\Common\xe2\x02\x1b\x43loudidl\\Common\\GPBMetadata\xea\x02\x10\x43loudidl::Commonb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'common.role_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023com.cloudidl.commonB\tRoleProtoH\002P\001Z)github.com/unionai/cloud/gen/pb-go/common\242\002\003CCX\252\002\017Cloudidl.Common\312\002\017Cloudidl\\Common\342\002\033Cloudidl\\Common\\GPBMetadata\352\002\020Cloudidl::Common'
  _ROLE.fields_by_name['id']._options = None
  _ROLE.fields_by_name['id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _ROLE.fields_by_name['permissions']._options = None
  _ROLE.fields_by_name['permissions']._serialized_options = b'\030\001'
  _globals['_ROLETYPE']._serialized_start=461
  _globals['_ROLETYPE']._serialized_end=720
  _globals['_ROLE']._serialized_start=117
  _globals['_ROLE']._serialized_end=412
  _globals['_ROLESPEC']._serialized_start=414
  _globals['_ROLESPEC']._serialized_end=458
# @@protoc_insertion_point(module_scope)



from union.remote._app_template_factory import HuggingFaceModelInfo, ShardConfig, VLLMShardArgs
from union.remote._remote import UnionRemote

__all__ = ["HuggingFaceModelInfo", "ShardConfig", "UnionRemote", "VLLMShardArgs"]



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from union.internal.identity import user_payload_pb2 as identity_dot_user__payload__pb2


class UserServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateUser = channel.unary_unary(
                '/cloudidl.identity.UserService/CreateUser',
                request_serializer=identity_dot_user__payload__pb2.CreateUserRequest.SerializeToString,
                response_deserializer=identity_dot_user__payload__pb2.CreateUserResponse.FromString,
                )
        self.GetUser = channel.unary_unary(
                '/cloudidl.identity.UserService/GetUser',
                request_serializer=identity_dot_user__payload__pb2.GetUserRequest.SerializeToString,
                response_deserializer=identity_dot_user__payload__pb2.GetUserResponse.FromString,
                )
        self.DeleteUser = channel.unary_unary(
                '/cloudidl.identity.UserService/DeleteUser',
                request_serializer=identity_dot_user__payload__pb2.DeleteUserRequest.SerializeToString,
                response_deserializer=identity_dot_user__payload__pb2.DeleteUserResponse.FromString,
                )
        self.ListUsers = channel.unary_unary(
                '/cloudidl.identity.UserService/ListUsers',
                request_serializer=identity_dot_user__payload__pb2.ListUsersRequest.SerializeToString,
                response_deserializer=identity_dot_user__payload__pb2.ListUsersResponse.FromString,
                )
        self.ListUsersCount = channel.unary_unary(
                '/cloudidl.identity.UserService/ListUsersCount',
                request_serializer=identity_dot_user__payload__pb2.ListUsersCountRequest.SerializeToString,
                response_deserializer=identity_dot_user__payload__pb2.ListUsersCountResponse.FromString,
                )
        self.GetUserAndGroupsForOrg = channel.unary_unary(
                '/cloudidl.identity.UserService/GetUserAndGroupsForOrg',
                request_serializer=identity_dot_user__payload__pb2.GetUserAndGroupsForOrgRequest.SerializeToString,
                response_deserializer=identity_dot_user__payload__pb2.GetUserAndGroupsForOrgResponse.FromString,
                )


class UserServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def CreateUser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetUser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteUser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListUsers(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListUsersCount(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetUserAndGroupsForOrg(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_UserServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'CreateUser': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateUser,
                    request_deserializer=identity_dot_user__payload__pb2.CreateUserRequest.FromString,
                    response_serializer=identity_dot_user__payload__pb2.CreateUserResponse.SerializeToString,
            ),
            'GetUser': grpc.unary_unary_rpc_method_handler(
                    servicer.GetUser,
                    request_deserializer=identity_dot_user__payload__pb2.GetUserRequest.FromString,
                    response_serializer=identity_dot_user__payload__pb2.GetUserResponse.SerializeToString,
            ),
            'DeleteUser': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteUser,
                    request_deserializer=identity_dot_user__payload__pb2.DeleteUserRequest.FromString,
                    response_serializer=identity_dot_user__payload__pb2.DeleteUserResponse.SerializeToString,
            ),
            'ListUsers': grpc.unary_unary_rpc_method_handler(
                    servicer.ListUsers,
                    request_deserializer=identity_dot_user__payload__pb2.ListUsersRequest.FromString,
                    response_serializer=identity_dot_user__payload__pb2.ListUsersResponse.SerializeToString,
            ),
            'ListUsersCount': grpc.unary_unary_rpc_method_handler(
                    servicer.ListUsersCount,
                    request_deserializer=identity_dot_user__payload__pb2.ListUsersCountRequest.FromString,
                    response_serializer=identity_dot_user__payload__pb2.ListUsersCountResponse.SerializeToString,
            ),
            'GetUserAndGroupsForOrg': grpc.unary_unary_rpc_method_handler(
                    servicer.GetUserAndGroupsForOrg,
                    request_deserializer=identity_dot_user__payload__pb2.GetUserAndGroupsForOrgRequest.FromString,
                    response_serializer=identity_dot_user__payload__pb2.GetUserAndGroupsForOrgResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'cloudidl.identity.UserService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class UserService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def CreateUser(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.identity.UserService/CreateUser',
            identity_dot_user__payload__pb2.CreateUserRequest.SerializeToString,
            identity_dot_user__payload__pb2.CreateUserResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetUser(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.identity.UserService/GetUser',
            identity_dot_user__payload__pb2.GetUserRequest.SerializeToString,
            identity_dot_user__payload__pb2.GetUserResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteUser(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.identity.UserService/DeleteUser',
            identity_dot_user__payload__pb2.DeleteUserRequest.SerializeToString,
            identity_dot_user__payload__pb2.DeleteUserResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListUsers(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.identity.UserService/ListUsers',
            identity_dot_user__payload__pb2.ListUsersRequest.SerializeToString,
            identity_dot_user__payload__pb2.ListUsersResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListUsersCount(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.identity.UserService/ListUsersCount',
            identity_dot_user__payload__pb2.ListUsersCountRequest.SerializeToString,
            identity_dot_user__payload__pb2.ListUsersCountResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetUserAndGroupsForOrg(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cloudidl.identity.UserService/GetUserAndGroupsForOrg',
            identity_dot_user__payload__pb2.GetUserAndGroupsForOrgRequest.SerializeToString,
            identity_dot_user__payload__pb2.GetUserAndGroupsForOrgResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: logs/dataplane/payload.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from union.internal.validate.validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1clogs/dataplane/payload.proto\x12\x17\x63loudidl.logs.dataplane\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x17validate/validate.proto\"o\n\x0bPodResource\x12%\n\tnamespace\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\tnamespace\x12\x1b\n\x04name\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x04name\x12\x1c\n\tcontainer\x18\x03 \x01(\tR\tcontainer\"\xda\x01\n\x1cTailTaskExecutionLogsRequest\x12Z\n\x0flogging_context\x18\x01 \x01(\x0b\x32\'.cloudidl.logs.dataplane.LoggingContextB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x0eloggingContext\x12;\n\x06source\x18\x03 \x01(\x0e\x32#.cloudidl.logs.dataplane.LogsSourceR\x06source\x12\x1b\n\tno_follow\x18\x04 \x01(\x08R\x08noFollowJ\x04\x08\x02\x10\x03\"\xeb\x01\n\x1dTailTaskExecutionLogsResponse\x12Q\n\x04logs\x18\x01 \x01(\x0b\x32;.cloudidl.logs.dataplane.TailTaskExecutionLogsResponse.LogsH\x00R\x04logs\x1am\n\x04Logs\x12\x18\n\x05lines\x18\x01 \x03(\tB\x02\x18\x01R\x05lines\x12K\n\x10structured_lines\x18\x02 \x03(\x0b\x32 .cloudidl.logs.dataplane.LogLineR\x0fstructuredLinesB\x08\n\x06result\"\xb8\x03\n\x0eLoggingContext\x12*\n\x0c\x63luster_name\x18\x03 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x0b\x63lusterName\x12:\n\x14kubernetes_namespace\x18\x04 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x13kubernetesNamespace\x12\x37\n\x13kubernetes_pod_name\x18\x05 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x11kubernetesPodName\x12\x43\n\x19kubernetes_container_name\x18\x06 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x17kubernetesContainerName\x12[\n\x1c\x65xecution_attempt_start_time\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x19\x65xecutionAttemptStartTime\x12W\n\x1a\x65xecution_attempt_end_time\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x17\x65xecutionAttemptEndTimeJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03\"\xf2\x01\n\x13\x43ontainerIdentifier\x12*\n\x0c\x63luster_name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x0b\x63lusterName\x12:\n\x14kubernetes_namespace\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x13kubernetesNamespace\x12\x37\n\x13kubernetes_pod_name\x18\x03 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x11kubernetesPodName\x12:\n\x19kubernetes_container_name\x18\x04 \x01(\tR\x17kubernetesContainerName\"\xb7\x02\n\x11\x43ontainerSelector\x12*\n\x0c\x63luster_name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x0b\x63lusterName\x12:\n\x14kubernetes_namespace\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x13kubernetesNamespace\x12;\n\x1akubernetes_pod_name_prefix\x18\x03 \x01(\tR\x17kubernetesPodNamePrefix\x12:\n\x19kubernetes_container_name\x18\x04 \x01(\tR\x17kubernetesContainerName\x12\x41\n\x1dkubernetes_pod_label_selector\x18\x05 \x01(\tR\x1akubernetesPodLabelSelector\"^\n\x0fLiveLogsOptions\x12$\n\x0elog_pod_status\x18\x01 \x01(\x08R\x0clogPodStatus\x12%\n\x0elog_timestamps\x18\x02 \x01(\x08R\rlogTimestamps\"\xd5\x03\n\x0fTailLogsRequest\x12L\n\tcontainer\x18\x01 \x01(\x0b\x32,.cloudidl.logs.dataplane.ContainerIdentifierH\x00R\tcontainer\x12[\n\x12\x63ontainer_selector\x18\x02 \x01(\x0b\x32*.cloudidl.logs.dataplane.ContainerSelectorH\x00R\x11\x63ontainerSelector\x12\x43\n\nstart_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x08\xfa\x42\x05\xb2\x01\x02\x08\x01R\tstartTime\x12\x35\n\x08\x65nd_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x07\x65ndTime\x12;\n\x06source\x18\x05 \x01(\x0e\x32#.cloudidl.logs.dataplane.LogsSourceR\x06source\x12T\n\x11live_logs_options\x18\x06 \x01(\x0b\x32(.cloudidl.logs.dataplane.LiveLogsOptionsR\x0fliveLogsOptionsB\x08\n\x06target\"\xa9\x01\n\x07LogLine\x12\x38\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimestamp\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message\x12J\n\noriginator\x18\x03 \x01(\x0e\x32*.cloudidl.logs.dataplane.LogLineOriginatorR\noriginator\"\xf0\x01\n\x08LogLines\x12\x18\n\x05lines\x18\x01 \x03(\tB\x02\x18\x01R\x05lines\x12\'\n\x0f\x63ontainer_index\x18\x02 \x01(\rR\x0e\x63ontainerIndex\x12T\n\tcontainer\x18\x03 \x01(\x0b\x32,.cloudidl.logs.dataplane.ContainerIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\tcontainer\x12K\n\x10structured_lines\x18\x04 \x03(\x0b\x32 .cloudidl.logs.dataplane.LogLineR\x0fstructuredLines\"a\n\x11LogContainersList\x12L\n\ncontainers\x18\x01 \x03(\x0b\x32,.cloudidl.logs.dataplane.ContainerIdentifierR\ncontainers\"F\n\rLogLinesBatch\x12\x35\n\x04logs\x18\x01 \x03(\x0b\x32!.cloudidl.logs.dataplane.LogLinesR\x04logs\"\xd9\x01\n\x10TailLogsResponse\x12V\n\ncontainers\x18\x01 \x01(\x0b\x32*.cloudidl.logs.dataplane.LogContainersListB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01H\x00R\ncontainers\x12Z\n\x0flog_lines_batch\x18\x03 \x01(\x0b\x32&.cloudidl.logs.dataplane.LogLinesBatchB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01H\x00R\rlogLinesBatchB\x0b\n\x04resp\x12\x03\xf8\x42\x01J\x04\x08\x02\x10\x03*6\n\x11LogLineOriginator\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x08\n\x04USER\x10\x01\x12\n\n\x06SYSTEM\x10\x02*F\n\nLogsSource\x12\x15\n\x11LIVE_OR_PERSISTED\x10\x00\x12\r\n\tLIVE_ONLY\x10\x01\x12\x12\n\x0ePERSISTED_ONLY\x10\x02\x42\xde\x01\n\x1b\x63om.cloudidl.logs.dataplaneB\x0cPayloadProtoH\x02P\x01Z1github.com/unionai/cloud/gen/pb-go/logs/dataplane\xa2\x02\x03\x43LD\xaa\x02\x17\x43loudidl.Logs.Dataplane\xca\x02\x17\x43loudidl\\Logs\\Dataplane\xe2\x02#Cloudidl\\Logs\\Dataplane\\GPBMetadata\xea\x02\x19\x43loudidl::Logs::Dataplaneb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'logs.dataplane.payload_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\033com.cloudidl.logs.dataplaneB\014PayloadProtoH\002P\001Z1github.com/unionai/cloud/gen/pb-go/logs/dataplane\242\002\003CLD\252\002\027Cloudidl.Logs.Dataplane\312\002\027Cloudidl\\Logs\\Dataplane\342\002#Cloudidl\\Logs\\Dataplane\\GPBMetadata\352\002\031Cloudidl::Logs::Dataplane'
  _PODRESOURCE.fields_by_name['namespace']._options = None
  _PODRESOURCE.fields_by_name['namespace']._serialized_options = b'\372B\004r\002\020\001'
  _PODRESOURCE.fields_by_name['name']._options = None
  _PODRESOURCE.fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _TAILTASKEXECUTIONLOGSREQUEST.fields_by_name['logging_context']._options = None
  _TAILTASKEXECUTIONLOGSREQUEST.fields_by_name['logging_context']._serialized_options = b'\372B\005\212\001\002\020\001'
  _TAILTASKEXECUTIONLOGSRESPONSE_LOGS.fields_by_name['lines']._options = None
  _TAILTASKEXECUTIONLOGSRESPONSE_LOGS.fields_by_name['lines']._serialized_options = b'\030\001'
  _LOGGINGCONTEXT.fields_by_name['cluster_name']._options = None
  _LOGGINGCONTEXT.fields_by_name['cluster_name']._serialized_options = b'\372B\004r\002\020\001'
  _LOGGINGCONTEXT.fields_by_name['kubernetes_namespace']._options = None
  _LOGGINGCONTEXT.fields_by_name['kubernetes_namespace']._serialized_options = b'\372B\004r\002\020\001'
  _LOGGINGCONTEXT.fields_by_name['kubernetes_pod_name']._options = None
  _LOGGINGCONTEXT.fields_by_name['kubernetes_pod_name']._serialized_options = b'\372B\004r\002\020\001'
  _LOGGINGCONTEXT.fields_by_name['kubernetes_container_name']._options = None
  _LOGGINGCONTEXT.fields_by_name['kubernetes_container_name']._serialized_options = b'\372B\004r\002\020\001'
  _CONTAINERIDENTIFIER.fields_by_name['cluster_name']._options = None
  _CONTAINERIDENTIFIER.fields_by_name['cluster_name']._serialized_options = b'\372B\004r\002\020\001'
  _CONTAINERIDENTIFIER.fields_by_name['kubernetes_namespace']._options = None
  _CONTAINERIDENTIFIER.fields_by_name['kubernetes_namespace']._serialized_options = b'\372B\004r\002\020\001'
  _CONTAINERIDENTIFIER.fields_by_name['kubernetes_pod_name']._options = None
  _CONTAINERIDENTIFIER.fields_by_name['kubernetes_pod_name']._serialized_options = b'\372B\004r\002\020\001'
  _CONTAINERSELECTOR.fields_by_name['cluster_name']._options = None
  _CONTAINERSELECTOR.fields_by_name['cluster_name']._serialized_options = b'\372B\004r\002\020\001'
  _CONTAINERSELECTOR.fields_by_name['kubernetes_namespace']._options = None
  _CONTAINERSELECTOR.fields_by_name['kubernetes_namespace']._serialized_options = b'\372B\004r\002\020\001'
  _TAILLOGSREQUEST.fields_by_name['start_time']._options = None
  _TAILLOGSREQUEST.fields_by_name['start_time']._serialized_options = b'\372B\005\262\001\002\010\001'
  _LOGLINES.fields_by_name['lines']._options = None
  _LOGLINES.fields_by_name['lines']._serialized_options = b'\030\001'
  _LOGLINES.fields_by_name['container']._options = None
  _LOGLINES.fields_by_name['container']._serialized_options = b'\372B\005\212\001\002\020\001'
  _TAILLOGSRESPONSE.oneofs_by_name['resp']._options = None
  _TAILLOGSRESPONSE.oneofs_by_name['resp']._serialized_options = b'\370B\001'
  _TAILLOGSRESPONSE.fields_by_name['containers']._options = None
  _TAILLOGSRESPONSE.fields_by_name['containers']._serialized_options = b'\372B\005\212\001\002\020\001'
  _TAILLOGSRESPONSE.fields_by_name['log_lines_batch']._options = None
  _TAILLOGSRESPONSE.fields_by_name['log_lines_batch']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_LOGLINEORIGINATOR']._serialized_start=3063
  _globals['_LOGLINEORIGINATOR']._serialized_end=3117
  _globals['_LOGSSOURCE']._serialized_start=3119
  _globals['_LOGSSOURCE']._serialized_end=3189
  _globals['_PODRESOURCE']._serialized_start=115
  _globals['_PODRESOURCE']._serialized_end=226
  _globals['_TAILTASKEXECUTIONLOGSREQUEST']._serialized_start=229
  _globals['_TAILTASKEXECUTIONLOGSREQUEST']._serialized_end=447
  _globals['_TAILTASKEXECUTIONLOGSRESPONSE']._serialized_start=450
  _globals['_TAILTASKEXECUTIONLOGSRESPONSE']._serialized_end=685
  _globals['_TAILTASKEXECUTIONLOGSRESPONSE_LOGS']._serialized_start=566
  _globals['_TAILTASKEXECUTIONLOGSRESPONSE_LOGS']._serialized_end=675
  _globals['_LOGGINGCONTEXT']._serialized_start=688
  _globals['_LOGGINGCONTEXT']._serialized_end=1128
  _globals['_CONTAINERIDENTIFIER']._serialized_start=1131
  _globals['_CONTAINERIDENTIFIER']._serialized_end=1373
  _globals['_CONTAINERSELECTOR']._serialized_start=1376
  _globals['_CONTAINERSELECTOR']._serialized_end=1687
  _globals['_LIVELOGSOPTIONS']._serialized_start=1689
  _globals['_LIVELOGSOPTIONS']._serialized_end=1783
  _globals['_TAILLOGSREQUEST']._serialized_start=1786
  _globals['_TAILLOGSREQUEST']._serialized_end=2255
  _globals['_LOGLINE']._serialized_start=2258
  _globals['_LOGLINE']._serialized_end=2427
  _globals['_LOGLINES']._serialized_start=2430
  _globals['_LOGLINES']._serialized_end=2670
  _globals['_LOGCONTAINERSLIST']._serialized_start=2672
  _globals['_LOGCONTAINERSLIST']._serialized_end=2769
  _globals['_LOGLINESBATCH']._serialized_start=2771
  _globals['_LOGLINESBATCH']._serialized_end=2841
  _globals['_TAILLOGSRESPONSE']._serialized_start=2844
  _globals['_TAILLOGSRESPONSE']._serialized_end=3061
# @@protoc_insertion_point(module_scope)



import inspect
import os
import site
import sys
from contextlib import contextmanager
from pathlib import Path

from flytekit.image_spec import default_builder

from union.ucimage import _image_builder

INTERNAL_APP_ENDPOINT_PATTERN_ENV_VAR = "INTERNAL_APP_ENDPOINT_PATTERN"


def _return_empty_str():
    return ""


@contextmanager
def patch_get_flytekit_for_pypi():
    """Mock out get_flytekit_for_pypi to return an empty string."""
    try:
        orig_get_flytekit_for_pypi = default_builder.get_flytekit_for_pypi
        default_builder.get_flytekit_for_pypi = _return_empty_str
        yield
        default_builder.get_flytekit_for_pypi = orig_get_flytekit_for_pypi
    except AttributeError:
        # Catch this error just in case `flytekit` changes the location of
        # get_flytekit_for_pypi
        yield


@contextmanager
def patch_get_unionai_for_pypi():
    """Mock out get_unionai_for_pypi to return an empty string."""
    orig_get_unionai_for_pypi = _image_builder.get_unionai_for_pypi
    _image_builder.get_unionai_for_pypi = _return_empty_str
    yield
    _image_builder.get_unionai_for_pypi = orig_get_unionai_for_pypi


@contextmanager
def patch_get_flytekit_and_union_for_pypi():
    """Patches both unionai and flyekit"""
    with patch_get_flytekit_for_pypi(), patch_get_unionai_for_pypi():
        yield


def _extract_files_loaded_from_cwd(cwd: Path) -> list[str]:
    """Look for all files loaded in sys.modules that is also in cwd."""
    cwd = os.fspath(cwd.absolute())
    loaded_modules = list(sys.modules)

    # Do not include site packages and anything in sys.prefix
    invalid_dirs = [site.getusersitepackages(), *site.getsitepackages(), sys.prefix]

    files_loaded_from_cwd = []
    for module_name in loaded_modules:
        try:
            module_file_path = inspect.getfile(sys.modules[module_name])
        except Exception:
            continue

        absolute_file_path = os.path.abspath(module_file_path)
        if not os.path.commonpath([absolute_file_path, cwd]) == cwd:
            continue

        is_invalid = any(
            os.path.commonpath([absolute_file_path, invalid_dir]) == invalid_dir for invalid_dir in invalid_dirs
        )
        if is_invalid:
            continue

        files_loaded_from_cwd.append(absolute_file_path)

    return files_loaded_from_cwd



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: workspace/workspace_instance_payload.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from union.internal.common import identifier_pb2 as common_dot_identifier__pb2
from union.internal.common import list_pb2 as common_dot_list__pb2
from flyteidl.admin import execution_pb2 as flyteidl_dot_admin_dot_execution__pb2
from flyteidl.core import literals_pb2 as flyteidl_dot_core_dot_literals__pb2
from union.internal.validate.validate import validate_pb2 as validate_dot_validate__pb2
from union.internal.workspace import workspace_definition_pb2 as workspace_dot_workspace__definition__pb2
from union.internal.workspace import workspace_instance_pb2 as workspace_dot_workspace__instance__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*workspace/workspace_instance_payload.proto\x12\x12\x63loudidl.workspace\x1a\x17\x63ommon/identifier.proto\x1a\x11\x63ommon/list.proto\x1a\x1e\x66lyteidl/admin/execution.proto\x1a\x1c\x66lyteidl/core/literals.proto\x1a\x17validate/validate.proto\x1a$workspace/workspace_definition.proto\x1a\"workspace/workspace_instance.proto\"\xe4\x02\n\x1dStartWorkspaceInstanceRequest\x12\x10\n\x03org\x18\x01 \x01(\tR\x03org\x12!\n\x07project\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x07project\x12\x1f\n\x06\x64omain\x18\x03 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01R\x06\x64omain\x12\x12\n\x04name\x18\x04 \x01(\tR\x04name\x12s\n\x17workspace_definition_id\x18\x05 \x01(\x0b\x32\x31.cloudidl.workspace.WorkspaceDefinitionIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x15workspaceDefinitionId\x12\x31\n\x04spec\x18\x06 \x01(\x0b\x32\x1d.flyteidl.admin.ExecutionSpecR\x04spec\x12\x31\n\x06inputs\x18\x07 \x01(\x0b\x32\x19.flyteidl.core.LiteralMapR\x06inputs\"v\n\x1eStartWorkspaceInstanceResponse\x12T\n\x12workspace_instance\x18\x01 \x01(\x0b\x32%.cloudidl.workspace.WorkspaceInstanceR\x11workspaceInstance\"^\n\x1bGetWorkspaceInstanceRequest\x12?\n\x02id\x18\x01 \x01(\x0b\x32/.cloudidl.workspace.WorkspaceInstanceIdentifierR\x02id\"t\n\x1cGetWorkspaceInstanceResponse\x12T\n\x12workspace_instance\x18\x01 \x01(\x0b\x32%.cloudidl.workspace.WorkspaceInstanceR\x11workspaceInstance\"\xf0\x02\n\x1dListWorkspaceInstancesRequest\x12\x36\n\x07request\x18\x01 \x01(\x0b\x32\x1c.cloudidl.common.ListRequestR\x07request\x12\x1b\n\x03org\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01H\x00R\x03org\x12H\n\x07project\x18\x03 \x01(\x0b\x32\".cloudidl.common.ProjectIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01H\x00R\x07project\x12u\n\x17workspace_definition_id\x18\x04 \x01(\x0b\x32\x31.cloudidl.workspace.WorkspaceDefinitionIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01H\x00R\x15workspaceDefinitionId\x12\'\n\tprincipal\x18\x05 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01H\x00R\tprincipalB\x10\n\tfilter_by\x12\x03\xf8\x42\x01\"\x8e\x01\n\x1eListWorkspaceInstancesResponse\x12V\n\x13workspace_instances\x18\x01 \x03(\x0b\x32%.cloudidl.workspace.WorkspaceInstanceR\x12workspaceInstances\x12\x14\n\x05token\x18\x02 \x01(\tR\x05token\"\xa7\x03\n\x1eWatchWorkspaceInstancesRequest\x12\x1b\n\x03org\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01H\x00R\x03org\x12H\n\x07project\x18\x02 \x01(\x0b\x32\".cloudidl.common.ProjectIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01H\x00R\x07project\x12u\n\x17workspace_definition_id\x18\x03 \x01(\x0b\x32\x31.cloudidl.workspace.WorkspaceDefinitionIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01H\x00R\x15workspaceDefinitionId\x12o\n\x15workspace_instance_id\x18\x04 \x01(\x0b\x32/.cloudidl.workspace.WorkspaceInstanceIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01H\x00R\x13workspaceInstanceId\x12\'\n\tprincipal\x18\x05 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01H\x00R\tprincipalB\r\n\x06target\x12\x03\xf8\x42\x01\"\xbf\x02\n\x1fWatchWorkspaceInstancesResponse\x12\x41\n\x0bstart_event\x18\x01 \x01(\x0b\x32\x1e.cloudidl.workspace.StartEventH\x00R\nstartEvent\x12\x44\n\x0cupdate_event\x18\x02 \x01(\x0b\x32\x1f.cloudidl.workspace.UpdateEventH\x00R\x0bupdateEvent\x12>\n\nstop_event\x18\x03 \x01(\x0b\x32\x1d.cloudidl.workspace.StopEventH\x00R\tstopEvent\x12J\n\x0estopping_event\x18\x04 \x01(\x0b\x32!.cloudidl.workspace.StoppingEventH\x00R\rstoppingEventB\x07\n\x05\x65vent\"\x7f\n\x1cStopWorkspaceInstanceRequest\x12I\n\x02id\x18\x01 \x01(\x0b\x32/.cloudidl.workspace.WorkspaceInstanceIdentifierB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01R\x02id\x12\x14\n\x05\x63\x61use\x18\x02 \x01(\tR\x05\x63\x61use\"\x1f\n\x1dStopWorkspaceInstanceResponseB\xd0\x01\n\x16\x63om.cloudidl.workspaceB\x1dWorkspaceInstancePayloadProtoH\x02P\x01Z,github.com/unionai/cloud/gen/pb-go/workspace\xa2\x02\x03\x43WX\xaa\x02\x12\x43loudidl.Workspace\xca\x02\x12\x43loudidl\\Workspace\xe2\x02\x1e\x43loudidl\\Workspace\\GPBMetadata\xea\x02\x13\x43loudidl::Workspaceb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'workspace.workspace_instance_payload_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.cloudidl.workspaceB\035WorkspaceInstancePayloadProtoH\002P\001Z,github.com/unionai/cloud/gen/pb-go/workspace\242\002\003CWX\252\002\022Cloudidl.Workspace\312\002\022Cloudidl\\Workspace\342\002\036Cloudidl\\Workspace\\GPBMetadata\352\002\023Cloudidl::Workspace'
  _STARTWORKSPACEINSTANCEREQUEST.fields_by_name['project']._options = None
  _STARTWORKSPACEINSTANCEREQUEST.fields_by_name['project']._serialized_options = b'\372B\004r\002\020\001'
  _STARTWORKSPACEINSTANCEREQUEST.fields_by_name['domain']._options = None
  _STARTWORKSPACEINSTANCEREQUEST.fields_by_name['domain']._serialized_options = b'\372B\004r\002\020\001'
  _STARTWORKSPACEINSTANCEREQUEST.fields_by_name['workspace_definition_id']._options = None
  _STARTWORKSPACEINSTANCEREQUEST.fields_by_name['workspace_definition_id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _LISTWORKSPACEINSTANCESREQUEST.oneofs_by_name['filter_by']._options = None
  _LISTWORKSPACEINSTANCESREQUEST.oneofs_by_name['filter_by']._serialized_options = b'\370B\001'
  _LISTWORKSPACEINSTANCESREQUEST.fields_by_name['org']._options = None
  _LISTWORKSPACEINSTANCESREQUEST.fields_by_name['org']._serialized_options = b'\372B\004r\002\020\001'
  _LISTWORKSPACEINSTANCESREQUEST.fields_by_name['project']._options = None
  _LISTWORKSPACEINSTANCESREQUEST.fields_by_name['project']._serialized_options = b'\372B\005\212\001\002\020\001'
  _LISTWORKSPACEINSTANCESREQUEST.fields_by_name['workspace_definition_id']._options = None
  _LISTWORKSPACEINSTANCESREQUEST.fields_by_name['workspace_definition_id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _LISTWORKSPACEINSTANCESREQUEST.fields_by_name['principal']._options = None
  _LISTWORKSPACEINSTANCESREQUEST.fields_by_name['principal']._serialized_options = b'\372B\004r\002\020\001'
  _WATCHWORKSPACEINSTANCESREQUEST.oneofs_by_name['target']._options = None
  _WATCHWORKSPACEINSTANCESREQUEST.oneofs_by_name['target']._serialized_options = b'\370B\001'
  _WATCHWORKSPACEINSTANCESREQUEST.fields_by_name['org']._options = None
  _WATCHWORKSPACEINSTANCESREQUEST.fields_by_name['org']._serialized_options = b'\372B\004r\002\020\001'
  _WATCHWORKSPACEINSTANCESREQUEST.fields_by_name['project']._options = None
  _WATCHWORKSPACEINSTANCESREQUEST.fields_by_name['project']._serialized_options = b'\372B\005\212\001\002\020\001'
  _WATCHWORKSPACEINSTANCESREQUEST.fields_by_name['workspace_definition_id']._options = None
  _WATCHWORKSPACEINSTANCESREQUEST.fields_by_name['workspace_definition_id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _WATCHWORKSPACEINSTANCESREQUEST.fields_by_name['workspace_instance_id']._options = None
  _WATCHWORKSPACEINSTANCESREQUEST.fields_by_name['workspace_instance_id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _WATCHWORKSPACEINSTANCESREQUEST.fields_by_name['principal']._options = None
  _WATCHWORKSPACEINSTANCESREQUEST.fields_by_name['principal']._serialized_options = b'\372B\004r\002\020\001'
  _STOPWORKSPACEINSTANCEREQUEST.fields_by_name['id']._options = None
  _STOPWORKSPACEINSTANCEREQUEST.fields_by_name['id']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_STARTWORKSPACEINSTANCEREQUEST']._serialized_start=272
  _globals['_STARTWORKSPACEINSTANCEREQUEST']._serialized_end=628
  _globals['_STARTWORKSPACEINSTANCERESPONSE']._serialized_start=630
  _globals['_STARTWORKSPACEINSTANCERESPONSE']._serialized_end=748
  _globals['_GETWORKSPACEINSTANCEREQUEST']._serialized_start=750
  _globals['_GETWORKSPACEINSTANCEREQUEST']._serialized_end=844
  _globals['_GETWORKSPACEINSTANCERESPONSE']._serialized_start=846
  _globals['_GETWORKSPACEINSTANCERESPONSE']._serialized_end=962
  _globals['_LISTWORKSPACEINSTANCESREQUEST']._serialized_start=965
  _globals['_LISTWORKSPACEINSTANCESREQUEST']._serialized_end=1333
  _globals['_LISTWORKSPACEINSTANCESRESPONSE']._serialized_start=1336
  _globals['_LISTWORKSPACEINSTANCESRESPONSE']._serialized_end=1478
  _globals['_WATCHWORKSPACEINSTANCESREQUEST']._serialized_start=1481
  _globals['_WATCHWORKSPACEINSTANCESREQUEST']._serialized_end=1904
  _globals['_WATCHWORKSPACEINSTANCESRESPONSE']._serialized_start=1907
  _globals['_WATCHWORKSPACEINSTANCESRESPONSE']._serialized_end=2226
  _globals['_STOPWORKSPACEINSTANCEREQUEST']._serialized_start=2228
  _globals['_STOPWORKSPACEINSTANCEREQUEST']._serialized_end=2355
  _globals['_STOPWORKSPACEINSTANCERESPONSE']._serialized_start=2357
  _globals['_STOPWORKSPACEINSTANCERESPONSE']._serialized_end=2388
# @@protoc_insertion_point(module_scope)
